@@ -44,48 +44,43 @@ fi
 
 
 
-echo -n "Checking pkg-config --cflags lua[5.1]..."
-if pkg-config --cflags lua5.1 >/dev/null 2>&1; then
-	echo "lua5.1"
-	lua_cflags="pkg-config --cflags lua5.1"	
-	lua_libs="pkg-config --libs lua5.1"	
-elif pkg-config --cflags lua5.2 >/dev/null 2>&1; then
-	echo "lua5.2"
-	lua_cflags="pkg-config --cflags lua5.2"
-	lua_libs="pkg-config --libs lua5.2"
-elif pkg-config --cflags lua >/dev/null 2>&1; then
-	echo "lua"
-	lua_cflags="pkg-config --cflags lua"
-	lua_libs="pkg-config --libs lua"
-elif pkg-config --cflags lua-5.1 >/dev/null 2>&1; then
-	echo "lua-5.1"
-	lua_cflags="pkg-config --cflags lua-5.1"
-	lua_libs="pkg-config --libs lua-5.1"
-elif pkg-config --cflags lua-5.2 >/dev/null 2>&1; then
-	echo "lua-5.2"
-	lua_cflags="pkg-config --cflags lua-5.2"
-	lua_libs="pkg-config --libs lua-5.2"
+echo -n "Checking pkg-config --cflags lua[5.1|5.2|51|52|jit]..."
+if [ "x$LUA" = "x" ]; then
+	lua_ver="lua5.1 lua5.2 lua lua-5.1 lua-5.2 lua51 lua52 luajit"
 else
-	echo "failed: no package lua/lua5.1/lua5.2/lua-5.1/lua-5.2"
+	lua_ver=$LUA
+fi
+
+for v in $lua_ver; do
+	if pkg-config --cflags "$v" >/dev/null 2>&1; then
+		echo "$v"
+		lua_cflags="pkg-config --cflags $v"
+		lua_libs="pkg-config --libs $v"
+		break
+	fi
+done
+
+if test "x$lua_libs" = "x"; then
+	echo "failed: no package lua/lua5.1/lua5.2/lua-5.1/lua-5.2/lua51/lua52"
 	echo "Please install lua development package."
 	exit 1
 fi
 
 
-echo -n "Checking sdl-config..."
+echo -n "Checking sdl2-config..."
 if ! sdl2-config --version >/dev/null 2>&1; then
 	if ! sdl-config --version >/dev/null 2>&1; then
-		echo "error: no sdl-config in \$PATH."
+		echo "error: no sdl-config/sdl2-config in \$PATH."
 		echo "Please install sdl, sdl_ttf, sdl_mixer and sdl_image development packages."
 		exit 1
 	fi
-	echo "ok"
+	echo "no, using SDL 1.xx"
 	sdl_config="sdl-config"
-	sdl_libs="-lSDL_ttf -lSDL_mixer -lSDL_image"
+	sdl_libs="-lSDL_ttf -lSDL_mixer -lSDL_image -lm"
 else
-	echo "SDL 2.0"
+	echo "ok"
 	sdl_config="sdl2-config"
-	sdl_libs="-lSDL2_ttf -lSDL2_mixer -lSDL2_image"
+	sdl_libs="-lSDL2_ttf -lSDL2_mixer -lSDL2_image -lm"
 fi
 
 echo -n "Checking sdl-config --cflags..."
@@ -95,7 +90,8 @@ if ! $sdl_config --cflags  >/dev/null 2>&1; then
 fi
 echo "ok"
 
-ops=`$lua_cflags`
+ops="$CPPFLAGS $CFLAGS $LDFLAGS"
+ops=$ops" "`$lua_cflags`
 ops=$ops" "`$lua_libs`
 
 echo -n "Looking for compiler..."
@@ -149,7 +145,7 @@ echo -n "Checking iconv...("
 echo -n "$cc /tmp/iconv-test.c -o iconv-test)..."
 
 if $cc /tmp/iconv-test.c -o /tmp/iconv-test >/dev/null 2>&1; then
-	CFLAGS="$CFLAGS -D_HAVE_ICONV"
+	CFLAGS="$CFLAGS -D_HAVE_ICONV -DLIBICONV_PLUG" # force FreeBSD to use iconv.h from base
 	echo "ok"
 elif $cc /tmp/iconv-test.c -liconv -o /tmp/iconv-test  >/dev/null 2>&1; then
 	CFLAGS="$CFLAGS -D_HAVE_ICONV"
@@ -200,8 +196,13 @@ echo "LUA_LFLAGS=\$(shell $lua_libs)" >> config.make
 echo "SDL_CFLAGS=\$(shell $sdl_config --cflags)" >> config.make
 echo "SDL_LFLAGS=\$(shell $sdl_config --libs) $sdl_libs" >> config.make
 echo "ok"
-echo -n "Choose installation mode. Standalone(1) or system(2) [1]: "
-read ans
+if [ "x$PREFIX" = "x" ]; then
+	echo -n "Choose installation mode. Standalone(1) or system(2) [1]: "
+	read ans
+else
+	ans="2"
+fi
+
 if [ "x$ans" = "x1" -o "x$ans" = "x" ]; then
 	echo " * Standalone version"
 	rm -f Rules.make
@@ -212,8 +213,12 @@ if [ "x$ans" = "x1" -o "x$ans" = "x" ]; then
 	echo "    \$ make"
 	echo "    \$ ./sdl-instead"
 elif [ "x$ans" = "x2" ]; then
-	echo -n "Enter prefix path [/usr/local]: "
-	read ans
+	if [ "x$PREFIX" = "x" ]; then
+		echo -n "Enter prefix path [/usr/local]: "
+		read ans
+	else
+		ans="$PREFIX"
+	fi
 
 	if [ "x$ans" = "x" ]; then
 		prefix="/usr/local"
@@ -227,15 +232,15 @@ elif [ "x$ans" = "x2" ]; then
 	echo " * System version with prefix: $prefix"
 
 	echo "PREFIX=$prefix" >> config.make
-	echo "BIN=\$(DESTDIR)\$(PREFIX)/bin/" >> config.make 
-	echo "DATAPATH=\$(DESTDIR)\$(PREFIX)/share/instead" >> config.make
+	echo "BIN=\$(PREFIX)/bin/" >> config.make 
+	echo "DATAPATH=\$(PREFIX)/share/instead" >> config.make
 	echo "STEADPATH=\$(DATAPATH)/stead" >> config.make
 	echo "THEMESPATH=\$(DATAPATH)/themes" >> config.make
 	echo "GAMESPATH=\$(DATAPATH)/games" >> config.make
-	echo "ICONPATH=\$(DESTDIR)\$(PREFIX)/share/pixmaps" >> config.make
-	echo "DOCPATH=\$(DESTDIR)\$(PREFIX)/share/doc/instead" >> config.make
+	echo "ICONPATH=\$(PREFIX)/share/pixmaps" >> config.make
+	echo "DOCPATH=\$(PREFIX)/share/doc/instead" >> config.make
 	echo "LANGPATH=\$(DATAPATH)/lang" >> config.make
-	echo "MANPATH=\$(DESTDIR)\$(PREFIX)/share/man/man6" >> config.make
+	echo "MANPATH=\$(PREFIX)/share/man/man6" >> config.make
 
 	echo "Ok. We are ready to build and install. Use these commands:"
 	echo "    \$ make"
