@@ -1,6 +1,10 @@
+#include <SDL_config.h>
 #ifdef _WIN32_WCE
 #include "wince.h"
 #endif
+#ifdef __IPHONEOS__
+#include "ios.h"
+#endif
 #include <lua.h>
 #include <lauxlib.h>
 #include <lualib.h>
@@ -8,12 +12,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
-// #include <libintl.h>
+/* #include <libintl.h> */
 #include <unistd.h>
-#include "sdl_iconv.h"
 
+#include "sdl_iconv.h"
 #include <limits.h>
-// #include <pwd.h>
+/* #include <pwd.h> */
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -25,4 +29,7 @@
 #ifndef S60
 #include <libgen.h>
 #endif
+#ifdef S60
+#include "snprintf.h"
+#endif
 #include <math.h>
