@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef __GAME_H__
 #define __GAME_H__
 #include <SDL_mixer.h>
@@ -16,9 +40,9 @@ extern int	game_running;
 extern int	game_theme_changed;
 
 extern int 	nosound_sw;
-extern int 	alsa_sw;
 extern int	debug_sw;
 extern int 	fullscreen_sw;
+extern int 	hires_sw;
 extern int 	window_sw;
 extern int	nopause_sw;
 extern int 	game_own_theme; /* current game has own theme */
@@ -26,8 +50,12 @@ extern char	*games_sw;
 extern char 	*err_msg; /* last error message */
 extern char 	game_cwd[PATH_MAX]; /* current game cwd */
 extern char 	*curgame_dir;
+extern int	vsync_sw;
+extern int	resizable_sw;
+extern int	standalone_sw;
 
 extern char	*game_local_stead_path(void);
+extern char	*appdir(void);
 
 extern char 	*game_local_games_path(int cr);
 extern char 	*game_tmp_path(void);
@@ -37,12 +65,14 @@ extern int 	game_init(const char *game);
 
 extern int 	game_loop(void);
 extern void 	game_done(int err);
+extern char	*game_reset_name(void);
 
-extern int 	game_load_theme(const char *path);
-extern int 	game_apply_theme(void);
-extern int game_use_theme(void);
-extern void game_release_theme(void);
-extern int game_reset(void);
+extern int	game_load_theme(const char *path);
+extern int	game_apply_theme(void);
+extern int	game_use_theme(void);
+extern void	game_release_theme(void);
+extern int	game_reset(void);
+extern int	game_cfg_save(void);
 
 extern void 	game_music_player(void);
 extern void	game_stop_mus(int ms);
@@ -51,21 +81,30 @@ extern int 	game_change_vol(int d, int val);
 extern int 	game_change_hz(int hz);
 
 extern int 	games_lookup(const char *path);
+extern struct game *game_lookup(const char *name);
+
+extern int	is_game(const char *games, const char *name);
 extern int	games_remove(int nr);
 extern int	games_replace(const char *path, const char *dir);
 extern int 	games_rename(void);
 
 extern void 	game_err_msg(const char *s);
+extern void 	game_res_err_msg(const char *s, int alert);
 extern int 	game_error(void);
 
 extern int	game_restart(void);
 extern int	game_select(const char *name);
-extern int	game_cmd(char *cmd, int click);
+
+#define GAME_CMD_CLICK 1
+#define GAME_CMD_FILE  2
+#define GAME_CMD_NOHL  4
+
+extern int	game_cmd(char *cmd, int flags);
 
 extern void	game_menu(int nr); /* select and show menu */
-extern void	game_menu_box(int show, const char *txt); /* show menu */
+extern int	game_menu_box(int show, const char *txt); /* show menu */
 extern int	game_menu_box_wh(const char *txt, int *w, int *h);
-extern void	game_menu_box_width(int show, const char *txt, int width);
+extern int	game_menu_box_width(int show, const char *txt, int width);
 
 extern void mouse_reset(int hl);
 extern int mouse_restore(void);
@@ -97,8 +136,9 @@ extern void sounds_free(void);
 extern const char *sound_channel(int i);
 extern void *sound_get(const char *fname);
 extern void sound_put(void *sn);
+extern int game_tag_valid(const char *p);
 
-void game_gfx_commit(void);
+void game_gfx_commit(int sync);
 
 #define CURSOR_CLEAR -1
 #define CURSOR_OFF    0
@@ -106,10 +146,14 @@ void game_gfx_commit(void);
 #define CURSOR_DRAW   2
 
 extern void 	game_cursor(int on); /* must be called with -1 before gfx change and 1 after, 2 - not update */
+extern int game_cursor_show;
 
 struct game {
 	char *path;
 	char *name;
+	char *author;
+	char *version;
+	char *info;
 	char *dir;
 	int idf;
 };
