@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef __GRAPHICS_H__
 #define __GRAPHICS_H__
 #include <SDL_version.h>
@@ -43,15 +67,7 @@ typedef struct {
 #define STYLE_ITALIC	0x02
 #define STYLE_UNDERLINE	0x04
 
-static inline color_t gfx_col(int r, int g, int b)
-{
-	color_t col;
-	col.r = r;
-	col.g = g;
-	col.b = b;
-	col.a = 0;
-	return col;
-}
+extern color_t gfx_col(int r, int g, int b);
 
 extern int 	gfx_init(void);
 extern void 	gfx_done(void);
@@ -68,18 +84,32 @@ extern void 	gfx_clip(int x, int y, int w, int h);
 extern void	gfx_getclip(int *x, int *y, int *w, int *h);
 extern int 	gfx_width;
 extern int 	gfx_height;
+extern int 	gfx_fs;
 extern int	gfx_video_init(void);
 extern int 	gfx_set_mode(int w, int h, int fs);
 extern int 	gfx_get_mode(int n, int *w, int *h);
-extern int	gfx_get_max_mode(int *w, int *h);
+extern int	gfx_get_token(const char *ptr, char **eptr, char **val, int *sp);
+
+#define MODE_ANY 0
+#define MODE_H 1
+#define MODE_V 2
+
+extern int	gfx_get_max_mode(int *w, int *h, int o);
+extern int	gfx_check_mode(int w, int h);
+
+extern int	gfx_set_title(const char *title);
+extern int	gfx_set_icon(img_t icon);
 
 extern int	gfx_next_mode(int *w, int *h);
 extern int	gfx_prev_mode(int *w, int *h);
 extern void	gfx_update(int x, int y, int w, int h);
 extern void	gfx_video_done(void);
 extern void	gfx_clear(int x, int y, int w, int h);
+extern void	gfx_resize(int w, int h);
+
 extern void	gfx_copy_from(img_t p, int x, int y, int width, int height, img_t to, int xx, int yy);
-extern void	gfx_draw(img_t pixmap, int x, int y);
+extern void	gfx_draw(img_t p, int x, int y);
+extern void	gfx_copy(img_t p, int x, int y);
 extern void	gfx_draw_wh(img_t p, int x, int y, int w, int h);
 extern img_t	gfx_grab_screen(int x, int y, int w, int h);
 extern img_t 	gfx_new(int w, int h);
@@ -89,7 +119,9 @@ extern int	gfx_img_w(img_t pixmap);
 extern int	gfx_img_h(img_t pixmap);
 extern img_t	gfx_combine(img_t src, img_t dst);
 extern void	gfx_set_alpha(img_t src, int alpha);
-extern void	gfx_unset_alpha(img_t src);
+extern int	gfx_unset_alpha(img_t src);
+extern void	gfx_set_colorkey(img_t src, color_t col);
+extern void	gfx_unset_colorkey(img_t src);
 
 extern img_t	gfx_alpha_img(img_t src, int alpha);
 extern img_t	gfx_display_alpha(img_t src);
@@ -100,7 +132,7 @@ extern void	gfx_draw_bg(img_t p, int x, int y, int width, int height);
 extern void	gfx_draw_from(img_t p, int x, int y, int width, int height, img_t to, int xx, int yy);
 extern void	gfx_compose_from(img_t p, int x, int y, int width, int height, img_t to, int xx, int yy);
 
-extern void	gfx_cursor(int *xp, int *yp);
+extern int	gfx_cursor(int *xp, int *yp);
 extern void	gfx_warp_cursor(int x, int y);
 extern void	gfx_change_screen(img_t src, int steps);
 extern int	gfx_fading(void);
@@ -117,9 +149,10 @@ extern img_t	fnt_render(fnt_t fn, const char *p, color_t col);
 
 extern void 	txt_draw(fnt_t fnt, const char *txt, int x, int y, color_t col);
 extern void 	txt_size(fnt_t fnt, const char *txt, int *w, int *h);
-// extern int 		txt_width(fnt_t fnt, const char *txt);
 
-//extern layout_t txt_layout(fnt_t fn, char *txt, int width, int height);
+/* extern int 		txt_width(fnt_t fnt, const char *txt);
+extern layout_t txt_layout(fnt_t fn, char *txt, int width, int height); */
+
 extern layout_t txt_layout(fnt_t fn, int align, int width, int height);
 extern void	txt_layout_add(layout_t lay, char *txt);
 extern void	txt_layout_set(layout_t lay, char *txt);
@@ -130,7 +163,7 @@ extern xref_t 	txt_layout_xref(layout_t lay, int x, int y);
 extern void 	txt_layout_color(layout_t lay, color_t fg);
 extern fnt_t 	txt_layout_font(layout_t lay);
 extern void 	txt_layout_font_height(layout_t lay, float height);
-
+extern textbox_t	txt_layout_box(layout_t lay);
 extern void 	txt_layout_link_color(layout_t lay, color_t link);
 extern void 	txt_layout_active_color(layout_t lay, color_t link);
 extern void	txt_layout_link_style(layout_t lay, int style);
@@ -176,7 +209,9 @@ extern int	xref_get_active(xref_t x);
 
 extern xref_t	xref_next(xref_t x);
 extern xref_t	xref_prev(xref_t x);
+
 extern int 	xref_position(xref_t x, int *xc, int *yc);
+extern int 	xref_valid(xref_t x);
 
 extern xref_t	txt_layout_xrefs(layout_t lay);
 
