@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include "externals.h"
 #include "internals.h"
 #include "idf.h"
@@ -13,7 +37,7 @@ typedef struct _idfd_t {
 
 
 struct _idff_t {
-	struct list_head list;
+	struct list_node list;
 	idfd_t		*dir;
 	unsigned long pos;
 	FILE		*fd;
@@ -27,7 +51,7 @@ static void free_idfd(void *p)
 
 	while (!list_empty(&dir->list)) {
 		idff_t idff;
-		idff = (idff_t)(dir->list.next);
+		idff = list_top(&dir->list, struct _idff_t, list);
 		idf_close(idff);
 	}
 
@@ -38,6 +62,7 @@ struct _idf_t {
 	unsigned long size;
 	FILE	*fd;
 	char 	*path;
+	char	cwd[PATH_MAX];
 	cache_t	dir;
 	int		idfonly;
 };
@@ -103,6 +128,19 @@ int	idf_magic(const char *fname)
 		return 1;
 	return 0;
 }
+int idf_setdir(idf_t idf, const char *path)
+{
+	if (idf && path)
+		strcpy(idf->cwd, path);
+	return 0;
+}
+
+char *idf_getdir(idf_t idf)
+{
+	if (!idf)
+		return NULL;
+	return idf->cwd;
+}
 
 idf_t idf_init(const char *fname)
 {
@@ -112,12 +150,14 @@ idf_t idf_init(const char *fname)
 	idf_t idf = malloc(sizeof(struct _idf_t));
 	if (!idf)
 		return NULL;
+	memset(idf, 0, sizeof(struct _idf_t));
 	idf->path = strdup(fname);
 	if (!idf->path)
 		goto err;
 	idf->idfonly = 0;
 	idf->fd = fopen(fp, "rb");
 	idf->dir = cache_init(-1, free_idfd);
+	idf->cwd[0] = 0;
 	if (!idf->fd || !idf->dir)
 		goto err;
 	if (fseek(idf->fd, 0, SEEK_END))
@@ -156,13 +196,13 @@ idf_t idf_init(const char *fname)
 		e->offset = off;
 		e->size = size;
 		e->idf = idf;
-		INIT_LIST_HEAD(&e->list);
+		list_head_init(&e->list);
 		if (cache_add(idf->dir, name, e)) {
 			free(e);
 			goto err;
 		}
 		cache_forget(idf->dir, e); /* use like hash */
-//		fprintf(stderr,"Parsed: '%s' @ %ld, %ld\n", name, off, size);
+/*		fprintf(stderr,"Parsed: '%s' @ %ld, %ld\n", name, off, size); */
 		dir_size -= (1 + sz + 4 + 4);
 	}
 	return idf;
@@ -172,11 +212,17 @@ err:
 }
 
 typedef struct {
-	struct list_head list;
+	struct list_node list;
 	char *path;
 	long size;
+	struct list_head dir;
 } idf_item_t;
 
+typedef struct {
+	struct list_node list;
+	char *dname;
+} idf_dir_item_t;
+
 static int fcopy(FILE *to, const char *fname)
 {
 	int rc = -1;
@@ -192,7 +238,7 @@ static int fcopy(FILE *to, const char *fname)
 				break;
 			goto err;
 		}
-		if (fwrite(buff, 1, s, to) != s)
+		if (fwrite(buff, 1, s, to) != (size_t)s)
 			goto err;
 	}
 	rc = 0;
@@ -203,6 +249,7 @@ err:
 
 static int idf_tree(const char *path, struct list_head *list, const char *fname)
 {
+	idf_item_t *dir; 
 	DIR *d;
 	struct dirent *de;
 	if (!path)
@@ -212,19 +259,26 @@ static int idf_tree(const char *path, struct list_head *list, const char *fname)
 		if (!access(dirpath(path), R_OK) && fname) {
 			FILE *fd; idf_item_t *i; 
 			fd = fopen(dirpath(path), "rb");
-			i = malloc(sizeof(idf_item_t));
-			if (!i)
+			if (!fd) {
+				fprintf(stderr, "Can not open file: '%s'\n", path);
 				return -1;
-			INIT_LIST_HEAD(&i->list);
+			}
+			i = malloc(sizeof(idf_item_t));
+			if (!i) {
+				fclose(fd);
+				return -1;
+			}
+			/* list_head_init(&i->list); */
 			if (!(i->path = strdup(fname)))
 				goto err;
 			if (fseek(fd, 0, SEEK_END) < 0)
 				goto err;
 			if ((i->size = ftell(fd)) < 0)
 				goto err;
+			list_head_init(&i->dir);
 			fclose(fd);
 			fprintf(stderr, "Added file: '%s' size: %ld\n", path, i->size);
-			list_add(&i->list, list);
+			list_add(list, &i->list);
 			return 0;
 		err:
 			if (i->path)
@@ -234,10 +288,34 @@ static int idf_tree(const char *path, struct list_head *list, const char *fname)
 		}
 		return 0;
 	}
+
+	dir = malloc(sizeof(idf_item_t));
+	if (!dir)
+		goto err1;
+	if (fname) {
+		char *d = malloc(strlen(fname) + 2);
+		if (!d)
+			goto err2;
+		strcpy(d, fname); strcat(d, "/");
+		dir->path = d;
+	} else {
+		dir->path = strdup("/");
+	}
+	dir->size = 0;
+	list_head_init(&dir->dir);
+
 	while ((de = readdir(d))) {
 		char *p;
+		idf_dir_item_t *di;
+
 		if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, ".."))
 			continue;
+		di = malloc(sizeof(idf_dir_item_t));
+		if (di) {
+			di->dname = strdup(de->d_name);
+			list_add(&dir->dir, &di->list);
+			dir->size += strlen(de->d_name) + 1;
+		}
 		p = getfilepath(path, de->d_name);
 		if (p) {
 			char *pp = getfilepath(fname, de->d_name);
@@ -248,8 +326,15 @@ static int idf_tree(const char *path, struct list_head *list, const char *fname)
 			free(p);
 		}
 	}
+	list_add(list, &dir->list);
+	fprintf(stderr, "Added dir: '%s' size: %ld\n", dir->path, dir->size);
 	closedir(d);
 	return 0;
+err2:
+	free(dir);
+err1:
+	closedir(d);
+	return -1;
 }
 
 int idf_create(const char *file, const char *path)
@@ -259,7 +344,7 @@ int idf_create(const char *file, const char *path)
 	char *p;
 	unsigned long off = 0;
 	long dict_size = 0;
-	struct list_head *pos;
+	idf_item_t *pos = NULL;
 
 	LIST_HEAD(items);
 	p = strdup(path);
@@ -278,8 +363,8 @@ int idf_create(const char *file, const char *path)
 
 	free(p);
 
-	list_for_each(pos, &items) {
-		idf_item_t *it = (idf_item_t *)pos;
+	list_for_each(&items, pos, list) {
+		idf_item_t *it = pos;
 		dict_size += (1 + strlen(it->path) + 4 + 4);
 	}
 
@@ -292,9 +377,9 @@ int idf_create(const char *file, const char *path)
 		goto err;
 	off = 4 + 4 + dict_size;
 
-	list_for_each(pos, &items) {
+	list_for_each(&items, pos, list) {
 		unsigned char s;
-		idf_item_t *it = (idf_item_t *)pos;
+		idf_item_t *it = pos;
 		s = strlen(it->path);
 		if (fwrite(&s, 1, 1, fd) != 1)
 			goto err;
@@ -314,9 +399,18 @@ int idf_create(const char *file, const char *path)
 		off += it->size;
 	}
 
-	list_for_each(pos, &items) {
-		idf_item_t *it = (idf_item_t *)pos;
+	list_for_each(&items, pos, list) {
+		idf_item_t *it = pos;
 		char *p;
+
+		if (!list_empty(&it->dir)) { /* directory-file */
+			idf_dir_item_t *d = NULL;
+			list_for_each(&it->dir, d, list) {
+				fprintf(fd, "%s\n", d->dname);
+			}
+			continue;
+		}
+
 		p = getfilepath(path, it->path);
 		if (p) {
 			int rc = fcopy(fd, p);
@@ -333,7 +427,13 @@ err:
 		fprintf(stderr, "Error creating idf file...\n");
 
 	while (!list_empty(&items)) {
-		idf_item_t *it = (idf_item_t *)items.next;
+		idf_item_t *it = list_top(&items, idf_item_t, list);
+		while (!list_empty(&it->dir)) {
+			idf_dir_item_t *di = list_top(&it->dir, idf_dir_item_t, list);
+			free(di->dname);
+			list_del(&di->list);
+			free(di);
+		}
 		free(it->path);
 		list_del(&it->list);
 		free(it);
@@ -349,7 +449,7 @@ int idf_seek(idff_t fil, int offset, int whence)
 	idfd_t *dir = fil->dir;
 	switch (whence) {
 	case SEEK_SET:
-		if (offset < 0 || offset > dir->size) {
+		if (offset < 0 || (unsigned int)offset > dir->size) {
 			return -1;
 		}
 		fil->pos = offset;
@@ -402,8 +502,8 @@ int idf_read(idff_t fil, void *ptr, int size, int maxnum)
 
 		if (fread(ptr, size, 1, fil->fd) != 1)
 			break;
-//		fil->pos += size;
-		ptr += size;
+/*		fil->pos += size; */
+		ptr = (char *)ptr + size;
 		maxnum --;
 		rc ++;
 	}
@@ -443,7 +543,15 @@ static 	int idfrw_close(struct SDL_RWops *context)
 	}
 	return 0;
 }
-
+#if SDL_VERSION_ATLEAST(2,0,0)
+static Sint64 idfrw_size(struct SDL_RWops *context)
+{
+	idff_t fil = (idff_t)context->hidden.unknown.data1;
+	if (!fil || !fil->dir)
+		return -1;
+	return fil->dir->size;
+}
+#endif
 #if 0
 int idf_extract(idf_t idf, const char *fname)
 {
@@ -501,6 +609,7 @@ int idf_only(idf_t idf, int fl)
 
 idff_t idf_open(idf_t idf, const char *fname)
 {
+	char *rp;
 	idfd_t *dir = NULL;
 	idff_t fil = NULL;
 	char *p;
@@ -510,8 +619,13 @@ idff_t idf_open(idf_t idf, const char *fname)
 	if (!p)
 		return NULL;
 	tolow(p);
-	if (idf)
-		dir = cache_lookup(idf->dir, p);
+	if (idf) {
+		rp = getfilepath(idf->cwd, p);
+		if (rp) {
+			dir = cache_lookup(idf->dir, rp);
+			free(rp);
+		}
+	}
 	free(p);
 	if (!dir)
 		return NULL;
@@ -519,14 +633,14 @@ idff_t idf_open(idf_t idf, const char *fname)
 	fil = malloc(sizeof(struct _idff_t));
 	if (!fil)
 		return NULL;
-	INIT_LIST_HEAD(&fil->list);
+/*	list_head_init(&fil->list); */
 
 	fil->dir = dir;
 	fil->pos = 0;
 	fil->fd = fopen(dirpath(idf->path), "rb");
 	if (!fil->fd)
 		goto err;
-	list_add(&fil->list, &dir->list);
+	list_add(&dir->list, &fil->list);
 	return fil;
 err:
 	free(fil);
@@ -535,14 +649,39 @@ err:
 
 int idf_access(idf_t idf, const char *fname)
 {
+	char *rp;
 	idfd_t *dir = NULL;
-	if (idf)
-		dir = cache_lookup(idf->dir, fname);
+	if (idf) {
+		rp = getfilepath(idf->cwd, fname);
+		if (rp) {
+			dir = cache_lookup(idf->dir, rp);
+			free(rp);
+		}
+	}
 	if (!dir)
 		return -1;
 	return 0;
 }
 
+idff_t  idf_opendir(idf_t idf, const char *dname)
+{
+	return idf_open(idf, dname);
+}
+
+int idf_closedir(idff_t d)
+{
+	return idf_close(d);
+}
+
+char *idf_readdir(idff_t d)
+{
+	char buff[256];
+	char *p = idf_gets(d, buff, sizeof(buff) - 1);
+	if (!p)
+		return p;
+	return strdup(p);
+}
+
 char *idf_gets(idff_t idf, char *b, int size)
 {
 	int rc, rc2;
@@ -582,6 +721,9 @@ SDL_RWops *RWFromIdf(idf_t idf, const char *fname)
 	n = SDL_AllocRW();
 	if (!n)
 		goto err;
+#if SDL_VERSION_ATLEAST(2,0,0)
+	n->size = idfrw_size;
+#endif
 	n->seek = idfrw_seek;
 	n->read = idfrw_read;
 	n->close = idfrw_close;
