@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include "externals.h"
 #include "internals.h"
 #include "list.h"
@@ -72,17 +96,17 @@ static int traceback (lua_State *L)
 	return 1;
 }
 #endif
-static int docall (lua_State *L) 
+static int docall (lua_State *L, int narg) 
 {
 	int status;
 	int base = 0; 	
 	if (debug_sw) {
-		base = lua_gettop(L);  /* function index */
+		base = lua_gettop(L) - narg;  /* function index */
 		lua_pushcfunction(L, traceback);  /* push traceback function */
 		lua_insert(L, base);  /* put it under chunk and args */
 	}
 	busy ++;
-	status = lua_pcall(L, 0, LUA_MULTRET, base);
+	status = lua_pcall(L, narg, LUA_MULTRET, base);
 	busy --;
 	if (debug_sw)
 		lua_remove(L, base);  /* remove traceback function */
@@ -93,7 +117,7 @@ static int docall (lua_State *L)
 }
 
 static int dofile (lua_State *L, const char *name) {
-	int status = luaL_loadfile(L, name) || docall(L);
+	int status = luaL_loadfile(L, name) || docall(L, 0);
 	return report(L, status);
 }
 
@@ -110,15 +134,15 @@ static const char *idf_reader(lua_State *L, void *data, size_t *size)
 
 static int dofile_idf (lua_State *L, idff_t idf, const char *name) {
 #if LUA_VERSION_NUM >= 502
-	int status = lua_load(L, idf_reader, idf, name, "bt") || docall(L);
+	int status = lua_load(L, idf_reader, idf, name, "bt") || docall(L, 0);
 #else
-	int status = lua_load(L, idf_reader, idf, name) || docall(L);
+	int status = lua_load(L, idf_reader, idf, name) || docall(L, 0);
 #endif
 	return report(L, status);
 }
 
 static int dostring (lua_State *L, const char *s) {
-	int status = luaL_loadstring(L, s) || docall(L);
+	int status = luaL_loadstring(L, s) || docall(L, 0);
 	return report(L, status);
 }
 
@@ -196,6 +220,19 @@ int instead_iretval(int n)
 	return lua_tonumber(L, n - N);
 }
 
+char *instead_file_cmd(char *s)
+{
+	struct instead_args args[] = {
+		{ .val = NULL, .type = INSTEAD_STR },
+		{ .val = NULL, },
+	};
+	if (!s)
+		return NULL;
+	args[0].val = s;
+	instead_function("iface:cmd", args);
+	return instead_retval(0);
+}
+
 char *instead_cmd(char *s)
 {
 	struct instead_args args[] = {
@@ -353,6 +390,23 @@ char *togame(const char *s)
 }
 #endif
 
+static int instead_getargs (char **argv, int n)
+{
+	int i;
+
+	for (i = 0; i < n; i++) {
+		lua_pushstring(L, argv[i]);
+	}
+
+	lua_createtable(L, n, 0);
+	for (i = 0; i < n; i++) {
+		lua_pushstring(L, argv[i]);
+		lua_rawseti(L, -2, i + 1);
+	}
+	lua_setglobal(L, "arg");
+	return 0;
+}
+
 int instead_load(char *game)
 {
 	idff_t idf = idf_open(game_idf, game);
@@ -374,6 +428,23 @@ int instead_load(char *game)
 	return 0;
 }
 
+int instead_loadscript(char *name, int argc, char **argv, int exec)
+{
+	int status;
+	if (exec)
+		instead_getargs(argv, argc);
+	status = luaL_loadfile(L, name);
+	if (!status) {
+		if (exec) {
+			lua_insert(L, -(argc + 1));
+			status |= docall(L, argc);
+		}
+	}
+	status = report(L, status);
+	instead_clear();
+	return status;
+}
+
 typedef struct LoadF {
 	int extraline;
 	unsigned char byte;
@@ -384,7 +455,7 @@ typedef struct LoadF {
 } LoadF;
 
 static const char *getF (lua_State *L, void *ud, size_t *size) {
-	int i = 0;
+	unsigned int i = 0;
 	LoadF *lf = (LoadF *)ud;
 	(void)L;
 	if (lf->extraline) {
@@ -447,11 +518,11 @@ static int loadfile (lua_State *L, const char *filename, int enc) {
 	else
 		readstatus = idf_error(lf.idff);
 
-	if (filename) {
-		if (lf.f)
-			fclose(lf.f);  /* close file (even in case of errors) */
+	if (lf.f)
+		fclose(lf.f);  /* close file (even in case of errors) */
+	else
 		idf_close(lf.idff);
-	}
+
 	if (readstatus) {
 		lua_settop(L, fnameindex);  /* ignore results from `lua_load' */
 		return errfile(L, "read", fnameindex);
@@ -515,10 +586,15 @@ static int luaB_get_savepath(lua_State *L) {
 }
 
 static int luaB_get_gamepath(lua_State *L) {
-	char path[PATH_MAX];
+	char path[PATH_MAX * 2];
 	char *p = getdir(path, sizeof(path));
-	if (p)
-		unix_path(p);
+	if (!p)
+		return 0;
+	unix_path(p);
+	if (game_idf) {
+		strcat(p, "/");
+		strcat(p, curgame_dir);
+	}
 	lua_pushstring(L, p);
 	return 1;
 }
@@ -564,6 +640,34 @@ static int luaB_get_gamespath(lua_State *L) {
 	return 1;
 }
 
+static int luaB_get_realpath(lua_State *L) {
+	char realpath[PATH_MAX];
+	char outpath[PATH_MAX];
+	const char *path = luaL_optstring(L, 1, NULL);
+	if (!path)
+		return 0;
+	strncpy(realpath, path, sizeof(realpath));
+	realpath[sizeof(realpath) - 1] = 0;
+	unix_path(realpath);
+	path = getrealpath(realpath, outpath);
+	if (!path)
+		return 0;
+	lua_pushstring(L, outpath);
+	return 1;
+}
+
+extern char *instead_exec;
+
+static int luaB_get_exepath(lua_State *L) {
+	char instead_path[PATH_MAX];
+	if (instead_exec) {
+		strcpy(instead_path, instead_exec);
+		unix_path(instead_path);
+		lua_pushstring(L, instead_exec);
+		return 1;
+	}
+	return 0;
+}
 
 static void instead_timer_do(void *data)
 {
@@ -627,7 +731,8 @@ static int luaB_theme_var(lua_State *L) {
 	}
 	if (!val || !var)
 		return 0;
-	game_own_theme = 1;
+	if (!game_own_theme)
+		return 0;
 	if (!opt_owntheme)
 		return 0;
 	if (!strcmp(var, "scr.w") ||
@@ -641,10 +746,20 @@ static int luaB_theme_var(lua_State *L) {
 }
 
 static int luaB_theme_name(lua_State *L) {
-	if (game_own_theme && opt_owntheme)
-		lua_pushstring(L, ".");
-	else
-		lua_pushstring(L, curtheme_dir);
+	char *name;
+	if (game_own_theme && opt_owntheme) {
+		if (game_own_theme == 2) {
+			name = malloc(strlen(curtheme_dir[THEME_GAME]) + 2);
+			if (!name)
+				return 0;
+			sprintf(name, ".%s", curtheme_dir[THEME_GAME]);
+			lua_pushstring(L, name);
+			free(name);
+		} else {
+			lua_pushstring(L, ".");
+		}
+	} else
+		lua_pushstring(L, curtheme_dir[THEME_GLOBAL]);
 	return 1;
 }
 
@@ -656,29 +771,29 @@ static LIST_HEAD(sprites);
 static LIST_HEAD(fonts);
 
 typedef struct {
-	struct list_head list;
+	struct list_node list;
 	char	*name;
 	fnt_t	fnt;
 } _fnt_t;
 
 typedef struct {
-	struct list_head list;
+	struct list_node list;
 	char	*name;
 	img_t	img;
 } _spr_t;
 
 static void sprites_free(void)
 {
-//	fprintf(stderr, "sprites free \n");
+/*	fprintf(stderr, "sprites free \n"); */
 	while (!list_empty(&sprites)) {
-		_spr_t *sp = (_spr_t*)(sprites.next);
+		_spr_t *sp = list_top(&sprites, _spr_t, list);
 		free(sp->name);
 		cache_forget(gfx_image_cache(), sp->img);
 		list_del(&sp->list);
 		free(sp);
 	}
 	while (!list_empty(&fonts)) {
-		_fnt_t *fn = (_fnt_t*)(fonts.next);
+		_fnt_t *fn = list_top(&fonts, _fnt_t, list);
 		fnt_free(fn->fnt);
 		free(fn->name);
 		list_del(&fn->list);
@@ -690,12 +805,13 @@ static void sprites_free(void)
 
 static _spr_t *sprite_lookup(const char *name)
 {
-	struct list_head *pos;
+	_spr_t *pos = NULL;
 	_spr_t *sp;
-	list_for_each(pos, &sprites) {
+	list_for_each(&sprites, pos, list) {
 		sp = (_spr_t*)pos;
 		if (!strcmp(name, sp->name)) {
-			list_move(&sp->list, &sprites); // move it on head
+			list_del(&sp->list);
+			list_add(&sprites, &sp->list); /* move it on head */
 			return sp;
 		}
 	}
@@ -704,12 +820,13 @@ static _spr_t *sprite_lookup(const char *name)
 
 static _fnt_t *font_lookup(const char *name)
 {
-	struct list_head *pos;
+	_fnt_t *pos = NULL;
 	_fnt_t *fn;
-	list_for_each(pos, &fonts) {
+	list_for_each(&fonts, pos, list) {
 		fn = (_fnt_t*)pos;
 		if (!strcmp(name, fn->name)) {
-			list_move(&fn->list, &fonts); // move it on head
+			list_del(&fn->list);
+			list_add(&fonts, &fn->list); /* move it on head */
 			return fn;
 		}
 	}
@@ -722,7 +839,7 @@ static _spr_t *sprite_new(const char *name, img_t img)
 	sp = malloc(sizeof(_spr_t));
 	if (!sp)
 		return NULL;
-	INIT_LIST_HEAD(&sp->list);
+/*	INIT_LIST_HEAD(&sp->list); */
 	sp->name = strdup(name);
 	if (!sp->name) {
 		free(sp);
@@ -734,8 +851,8 @@ static _spr_t *sprite_new(const char *name, img_t img)
 		free(sp);
 		return NULL;
 	}
-//	fprintf(stderr, "added: %s\n", name);
-	list_add(&sp->list, &sprites);
+/*	fprintf(stderr, "added: %s\n", name); */
+	list_add(&sprites, &sp->list);
 	return sp;
 }
 
@@ -745,14 +862,14 @@ static _fnt_t *font_new(const char *name, fnt_t fnt)
 	fn = malloc(sizeof(_fnt_t));
 	if (!fn)
 		return NULL;
-	INIT_LIST_HEAD(&fn->list);
+/*	INIT_LIST_HEAD(&fn->list); */
 	fn->name = strdup(name);
 	if (!fn->name) {
 		free(fn);
 		return NULL;
 	}
 	fn->fnt = fnt;
-	list_add(&fn->list, &fonts);
+	list_add(&fonts, &fn->list);
 	return fn;
 }
 
@@ -802,11 +919,13 @@ static int luaB_load_sprite(lua_State *L) {
 		return 0;
 
 	img = gfx_load_image((char*)fname);
-	if (img)
-		img = gfx_display_alpha(img); /*speed up */
+
 	if (img)
 		theme_img_scale(&img);
 
+	if (img)
+		img = gfx_display_alpha(img); /*speed up */
+
 	if (!img)
 		goto err;
 
@@ -822,6 +941,7 @@ static int luaB_load_sprite(lua_State *L) {
 	lua_pushstring(L, key);
 	return 1;
 err:
+	game_res_err_msg(fname, debug_sw);
 	gfx_free_image(img);
 	return 0;
 }
@@ -1119,6 +1239,28 @@ err:
 	return 0;
 }
 
+static int luaB_colorkey_sprite(lua_State *L) {
+	img_t s;
+	color_t  col;
+
+	const char *src = luaL_optstring(L, 1, NULL);
+	const char *color = luaL_optstring(L, 2, NULL);
+
+	if (color)
+		gfx_parse_color(color, &col);
+
+	if (!src)
+		return 0;
+	s = cache_lookup(gfx_image_cache(), src);
+	if (!s)
+		return 0;
+	if (color)
+		gfx_set_colorkey(s, col);
+	else 
+		gfx_unset_colorkey(s);
+	return 0;
+}
+
 static int luaB_dup_sprite(lua_State *L) {
 	_spr_t *sp;
 	img_t s;
@@ -1366,37 +1508,26 @@ static int luaB_free_sprite(lua_State *L) {
 	return 1;
 }
 
-static int luaB_show_menu(lua_State *L) {
-	const char *menu = luaL_optstring(L, 1, NULL);
-	if (menu) {
-		if (!strcmp(menu, "save"))
-			menu_toggle(menu_save);
-		else if (!strcmp(menu, "load"))
-			menu_toggle(menu_load);
-		else if (!strcmp(menu, "quit") || !strcmp(menu, "exit"))
-			menu_toggle(menu_askquit);
-		else
-			menu_toggle(-1);
-		return 0;
-	}
-	menu_toggle(menu_main);
-	return 0;
-}
-
 static int luaB_stead_busy(lua_State *L) {
 	static unsigned long busy_time = 0;
 	int busy = lua_toboolean(L, 1);
 	if (busy) {
 		struct inp_event ev;
+		int dirty = 0;
 		memset(&ev, 0, sizeof(ev));
 		while (input(&ev, 0) == AGAIN);
-		if (ev.type == MOUSE_MOTION)
+		if (ev.type == MOUSE_MOTION) {
 			game_cursor(CURSOR_ON); /* to make all happy */
+			dirty = 1;
+		}
 		if (!busy_time)
 			busy_time = gfx_ticks();
-		if (gfx_ticks() - busy_time >= 750 && menu_visible() != menu_wait)
+		if (gfx_ticks() - busy_time >= 750 && menu_visible() != menu_wait) {
 			game_menu(menu_wait);
-		game_gfx_commit();
+			dirty = 1;
+		}
+		if (dirty)
+			game_gfx_commit(0);
 		return 0;
 	}
 	if (menu_visible() == menu_wait) {
@@ -1417,6 +1548,7 @@ static int luaB_free_font(lua_State *L) {
 		return 0;
 
 	list_del(&fn->list);
+	fnt_free(fn->fnt);
 	free(fn->name); free(fn);
 	lua_pushboolean(L, 1);
 	return 1;
@@ -1478,6 +1610,7 @@ static int luaB_channel_sound(lua_State *L) {
 static int luaB_mouse_pos(lua_State *L) {
 	int x = luaL_optnumber(L, 1, -1);
 	int y = luaL_optnumber(L, 2, -1);
+	int m;
 	float v = game_theme.scale;
 	if (x != -1 && y != -1) {
 		x *= v;
@@ -1486,12 +1619,30 @@ static int luaB_mouse_pos(lua_State *L) {
 		x = -1;
 		y = -1;
 	}
-	gfx_cursor(&x, &y);
+	m = gfx_cursor(&x, &y);
 	x = (x - game_theme.xoff) / v;
 	y = (y - game_theme.yoff) / v;
 	lua_pushnumber(L, x);
 	lua_pushnumber(L, y);
-	return 2;
+	lua_pushnumber(L, m);
+	return 3;
+}
+
+static int luaB_finger_pos(lua_State *L) {
+	int x, y;
+	float pressure;
+	float v = game_theme.scale;
+	const char *finger = luaL_optstring(L, 1, NULL);
+	if (!finger)
+		return 0;
+	if (finger_pos(finger, &x, &y, &pressure)) /* no finger */
+		return 0;
+	x = (x - game_theme.xoff) / v;
+	y = (y - game_theme.yoff) / v;
+	lua_pushnumber(L, x);
+	lua_pushnumber(L, y);
+	lua_pushnumber(L, pressure);
+	return 3;
 }
 
 extern int mouse_filter_delay;
@@ -1505,6 +1656,15 @@ static int luaB_mouse_filter(lua_State *L) {
 	return 1;
 }
 
+static int luaB_mouse_show(lua_State *L) {
+	int show = lua_toboolean(L, 1);
+	int ov = game_cursor_show;
+	if (lua_isboolean(L, 1))
+		game_cursor_show = show;
+	lua_pushboolean(L, ov);
+	return 1;
+}
+
 static int luaB_get_ticks(lua_State *L) {
 	lua_pushnumber(L, gfx_ticks());
 	return 1;
@@ -1652,11 +1812,35 @@ static int luaB_maxn (lua_State *L) {
 	return 1;
 }
 
+static int luaB_srandom(lua_State *L) {
+	mt_random_seed(luaL_optnumber(L, 1, time(NULL)));
+	return 0;
+}
+
+static int luaB_random(lua_State *L) {
+	lua_Number rt;
+	unsigned long r = 0;
+	long a = luaL_optnumber(L, 1, -1);
+	long b = luaL_optnumber(L, 2, -1);
+	r = mt_random();
+	if (a >=0 && b >= a) {
+		rt = a + (r % (b - a + 1));
+	} else if (a >=0 && b == -1) {
+		rt = (r % a) + 1;
+	} else {
+		rt = mt_random_double();
+	}
+	lua_pushnumber(L, rt);
+	return 1;
+}
+
 static const luaL_Reg base_funcs[] = {
 	{"doencfile", luaB_doencfile},
 	{"dofile", luaB_dofile},
 	{"print", luaB_print}, /* for some mystic, it is needed in win version (with -debug) */
 	{"table_get_maxn", luaB_maxn},
+	{"instead_random", luaB_random},
+	{"instead_srandom", luaB_srandom},
 
 	{"instead_readdir", dir_iter_factory},
 	{"instead_sound", luaB_is_sound},
@@ -1665,10 +1849,11 @@ static const luaL_Reg base_funcs[] = {
 	{"instead_steadpath", luaB_get_steadpath},
 	{"instead_themespath", luaB_get_themespath},
 	{"instead_gamespath", luaB_get_gamespath},
+	{"instead_realpath", luaB_get_realpath},
+	{"instead_exepath", luaB_get_exepath},
 	{"instead_timer", luaB_set_timer},
 	{"instead_theme_var", luaB_theme_var},
 	{"instead_theme_name", luaB_theme_name},
-	{"instead_menu_toggle", luaB_show_menu},
 	{"instead_busy", luaB_stead_busy},
 	{"instead_sound_load", luaB_load_sound},
 	{"instead_sound_free", luaB_free_sound},
@@ -1679,6 +1864,8 @@ static const luaL_Reg base_funcs[] = {
 
 	{"instead_mouse_pos", luaB_mouse_pos},
 	{"instead_mouse_filter", luaB_mouse_filter},
+	{"instead_mouse_show", luaB_mouse_show},
+	{"instead_finger_pos", luaB_finger_pos},
 
 	{"instead_font_load", luaB_load_font},
 	{"instead_font_free", luaB_free_font},
@@ -1694,6 +1881,7 @@ static const luaL_Reg base_funcs[] = {
 	{"instead_sprite_fill", luaB_fill_sprite},
 	{"instead_sprite_dup", luaB_dup_sprite},
 	{"instead_sprite_alpha", luaB_alpha_sprite},
+	{"instead_sprite_colorkey", luaB_colorkey_sprite},
 	{"instead_sprite_size", luaB_sprite_size},
 	{"instead_sprite_scale", luaB_scale_sprite},
 	{"instead_sprite_rotate", luaB_rotate_sprite},
@@ -1738,30 +1926,22 @@ static int instead_platform(void)
 	if (!L)
 		return 0;
 
-	snprintf(plat, sizeof(plat) - 1, "PLATFORM='UNIX'");
-
-#ifdef __APPLE__
+#if defined(IOS)
+	snprintf(plat, sizeof(plat) - 1, "PLATFORM='IOS'");
+#elif defined(__APPLE__)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='MACOSX'");
-#endif
-
-#ifdef _WIN32
-	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WIN32'");
-#endif
-
-#ifdef _WIN32_WCE
+#elif defined(_WIN32_WCE)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WINCE'");
-#endif
-
-#ifdef S60
+#elif defined(S60)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='S60'");
-#endif
-
-#ifdef ANDROID
+#elif defined(ANDROID)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='ANDROID'");
-#endif
-
-#ifdef MAEMO
+#elif defined(_WIN32)
+	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WIN32'");
+#elif defined(MAEMO)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='MAEMO'");
+#else
+	snprintf(plat, sizeof(plat) - 1, "PLATFORM='UNIX'");
 #endif
 
 	plat[sizeof(plat) - 1] = 0;
@@ -1772,21 +1952,24 @@ static int instead_platform(void)
 static int instead_package(const char *path)
 {
 	char *p;
+	char *stead_path;
+	stead_path = malloc(PATH_MAX * 5); /* game_cwd + STEAD_PATH and so on... */
+	if (!stead_path)
+		return -1;
+	strcpy(stead_path, "package.path=\"");
 #if defined(_WIN32_WCE)
-	char stead_path[PATH_MAX * 4] = "package.path=\""; /* wince have not cwd :) */
 	if (path) {
-		strcat(stead_path, path);
-		strcat(stead_path, "/?.lua");
-		strcat(stead_path, ";");
+		strcat(stead_path, path); /* wince have not cwd :) */
+		strcat(stead_path, "/?.lua;");
 	}
 #else
-	char stead_path[PATH_MAX] = "package.path=\"./?.lua;";
+	if (path)
+		strcat(stead_path, "./?.lua;");
 #endif
 	p = game_local_stead_path();
 	if (p) {
 		strcat(stead_path, p);
-		strcat(stead_path, "/?.lua");
-		strcat(stead_path, ";");
+		strcat(stead_path, "/?.lua;");
 	}
 
 	if (STEAD_PATH[0] != '/') {
@@ -1799,18 +1982,23 @@ static int instead_package(const char *path)
 	strcat(stead_path, "/?.lua");
 	strcat(stead_path, "\"");
 
+	if (standalone_sw) {
+		strcat(stead_path, "..';'..(package.path or '')");
+	}
 
 	instead_eval(stead_path); instead_clear();
+	free(stead_path);
 /*	putenv(stead_path); */
 	return 0;
 }
 
-int instead_init(const char *path)
+int instead_init_lua(const char *path)
 {
 	busy = 0;
 	setlocale(LC_ALL,"");
 	setlocale(LC_NUMERIC,"C"); /* to avoid . -> , in numbers */	
-//	strcpy(curcp, "UTF-8");
+	setlocale(LC_CTYPE,"C"); /* to avoid lower/upper problems */	
+/*	strcpy(curcp, "UTF-8"); */
 
 	/* initialize Lua */
 #if LUA_VERSION_NUM >= 502
@@ -1836,6 +2024,19 @@ int instead_init(const char *path)
 	instead_package(path);
 	instead_platform();
 	instead_lang();
+	if (debug_sw)
+		instead_eval("DEBUG=true"); instead_clear();
+	if (standalone_sw)
+		instead_eval("STANDALONE=true"); instead_clear();
+	srand(time(NULL));
+	mt_random_init();
+	return 0;
+}
+
+int instead_init(const char *path)
+{
+	if (instead_init_lua(path))
+		return -1;
 
 	if (dofile(L, dirpath(STEAD_PATH"/stead.lua"))) {
 		return -1;
@@ -1847,7 +2048,6 @@ int instead_init(const char *path)
 	}
 	/* cleanup Lua */
 	instead_clear();
-	srand(time(NULL));
 	return 0;
 }
 
@@ -1873,7 +2073,7 @@ int  instead_encode(const char *s, const char *d)
 	FILE *src;
 	FILE *dst;
 	size_t size;
-	int i = 0;
+	unsigned int i = 0;
 	unsigned char byte = 0xcc;
 	unsigned char buff[4096];
 
