@@ -1,14 +1,39 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include "externals.h"
 #include "internals.h"
 
 
 static int restart_needed = 0;
 static int games_menu_from = 0;
-static int themes_menu_from = 0;
+static int themes_menu_page = 0;
 
 static int cur_lang = 0;
 
 int cur_menu = menu_main;
+int top_menu = menu_main;
 
 char *UNKNOWN_ERROR = NULL;
 char *ERROR_MENU = NULL;
@@ -57,7 +82,8 @@ static char *slot_name(const char *path)
 		return NULL;
 	l = lookup_tag(path, "Name", "--");
 	if (l) {
-		if (!is_empty(l)) {
+		trunc_lines(l, 0);
+		if (!is_empty(l) && game_tag_valid(l)) {
 			int y;
 			char *m;
 			static char *months[] = {/* to work on every locale ;)*/
@@ -286,15 +312,32 @@ int games_menu_maxw(void)
 
 static void themes_menu(void)
 {
-	int i, n;
+	int i, n, m;
+	int type;
+	int count = themes_max(&type);
+	int pages = 0;
+
+	if (count > 0)
+		pages = (count - 1) / MENU_THEMES_MAX + 1;
+
+	if (themes_menu_page >= pages)
+		themes_menu_page = 0;
+
 	strcpy(menu_buff, SELECT_THEME_MENU);
-	if ((themes_nr - 1) / MENU_THEMES_MAX)
-		pages_menu(menu_buff, themes_menu_from / MENU_THEMES_MAX, (themes_nr - 1) / MENU_THEMES_MAX + 1, "themes", "\n");
-	for (i = themes_menu_from, n = 0; i < themes_nr && n < MENU_THEMES_MAX; i ++) {
+
+	if (pages > 1)
+		pages_menu(menu_buff, themes_menu_page, pages, "themes", "\n");
+
+	for (i = 0, n = 0, m = 0; i < themes_nr && n < MENU_THEMES_MAX; i ++) {
 		char tmp[PATH_MAX];
 		if (!themes[i].name[0]) /* empty */
 			continue;
-		if (curtheme_dir && !strcmp(themes[i].dir, curtheme_dir))
+		if (themes[i].type != type)
+			continue;
+		m ++;
+		if ((m - 1) / MENU_THEMES_MAX < themes_menu_page)
+			continue;
+		if (curtheme_dir[type] && !strcmp(themes[i].dir, curtheme_dir[type]))
 			snprintf(tmp, sizeof(tmp), "<l><a:/resume><b>%s</b></a></l>\n", themes[i].name);
 		else
 			snprintf(tmp, sizeof(tmp), "<l><a:%s>%s</a></l>\n", themes[i].dir, themes[i].name);
@@ -302,10 +345,10 @@ static void themes_menu(void)
 		n ++;
 	}
 
-	for(;n < MENU_THEMES_MAX && themes_nr > MENU_THEMES_MAX; n++) /* align h */
+	for(;n < MENU_THEMES_MAX && count > MENU_THEMES_MAX; n++) /* align h */
 		strcat(menu_buff, "\n");
 
-	if (!themes_nr)
+	if (!count)
 		sprintf(menu_buff, NOTHEMES_MENU, THEMES_PATH);
 	strcat(menu_buff, "\n");
 	strcat(menu_buff, BACK_MENU); 
@@ -314,17 +357,23 @@ static void themes_menu(void)
 int themes_menu_maxw(void)
 {
 	int i = 0;
-	int oldm = themes_menu_from;
+	int oldm = themes_menu_page;
 	int maxw = 0;
-	for (i = 0; i < themes_nr; i += MENU_THEMES_MAX) {
+	int pages = 0;
+	int count = themes_max(NULL);
+	if (count > 0)
+		pages = (count - 1) / MENU_THEMES_MAX + 1;
+	else
+		pages = 1;
+	for (i = 0; i < pages; i ++) {
 		int w;
-		themes_menu_from = i;
+		themes_menu_page = i;
 		themes_menu();
 		game_menu_box_wh(menu_buff, &w, NULL);
 		if (w > maxw)
 			maxw = w;
 	}
-	themes_menu_from = oldm;
+	themes_menu_page = oldm;
 	return maxw;
 }
 
@@ -341,22 +390,122 @@ static char *opt_get_mode(void)
 
 static int gtr = 0;
 static int menu_settings_num = 0;
+
+static void menu_strip_tag(const char *a, const char *b)
+{
+	char *p, *ep;
+	size_t len;
+	p = strstr(menu_buff, a);
+	if (!p)
+		return;
+	ep = strstr(p, b);
+	if (!ep)
+		return;
+/*	ep += strcspn(ep, "\n\r");
+	ep += strspn(ep, "\n\r"); */
+	ep += strlen(b);
+	len = strlen(ep);
+	memmove(p, ep, len);
+	p[len] = 0;
+}
+
+static void menu_remove_tag(const char *a, const char *b)
+{
+	char *p, *ep;
+	size_t len;
+	p = strstr(menu_buff, a);
+	if (!p)
+		return;
+	ep = p + strlen(a);
+	len = strlen(ep);
+	memmove(p, ep, len);
+	p[len] = 0;
+
+	ep = strstr(p, b);
+	if (!ep)
+		return;
+	p = ep;
+	ep += strlen(b);
+	len = strlen(ep);
+
+	memmove(p, ep, len);
+	p[len] = 0;
+}
+
 char *game_menu_gen(void)
 {
 	if (cur_menu == menu_main) {
 		strcpy(menu_buff, MAIN_MENU);
-	} else if (cur_menu == menu_about) {
-		snprintf(menu_buff, sizeof(menu_buff), ABOUT_MENU, VERSION);
+		if (standalone_sw) {
+			int count = 0, n;
+			menu_strip_tag("<?:select>", "</?>");
+			count = themes_count(THEME_GLOBAL);
+			if (curgame_dir && opt_owntheme) {
+				if ((n = themes_count(THEME_GAME)) > 0)
+					count = n;
+				else if (game_own_theme)
+					count = 1;
+			}
+			if (count <= 1)
+				menu_strip_tag("<?:themes>", "</?>");
+			else
+				menu_remove_tag("<?:themes>", "</?>");
+		} else {
+			menu_remove_tag("<?:select>", "</?>");
+			menu_remove_tag("<?:themes>", "</?>");
+		}
+	} else if (cur_menu == menu_about || cur_menu == menu_about_instead) {
+		struct game *g;
+		if (cur_menu == menu_about && curgame_dir && (g = game_lookup(curgame_dir))) {
+			char version[32];
+			char author[64];
+			char info[192];
+			char instead[64];
+
+			if (g->version)
+				snprintf(version, sizeof(version), "%s", g->version);
+			else
+				strcpy(version, "1.0");
+
+			if (g->author)
+				snprintf(author, sizeof(author), "\n%s", g->author);
+			else
+				strcpy(author, "");
+
+			if (g->info)
+				snprintf(info, sizeof(info), "\n\n%s", g->info);
+			else
+				strcpy(info, "");
+
+			if (!standalone_sw)
+				snprintf(instead, sizeof(instead), "<a:/about-instead>INSTEAD</a> | ");
+			else
+				strcpy(instead, "");
+			author[sizeof(author) - 1] = 0;
+			version[sizeof(version) - 1] = 0;
+			info[sizeof(info) - 1] = 0;
+			snprintf(menu_buff, sizeof(menu_buff), "%s - %s%s%s\n\n%s%s", 
+				g->name, version,
+				author, info,
+				instead, BACK_MENU);
+		} else {
+			snprintf(menu_buff, sizeof(menu_buff), ABOUT_MENU, VERSION);
+		}
 	} else if (cur_menu == menu_settings) {
 		char *just[JUST_MAX] = { FROM_THEME, OFF, ON };
 		char *kbd [KBD_MAX] = { KBD_MODE_SMART, KBD_MODE_LINKS, KBD_MODE_SCROLL };
+		int fsize = 100 + (10 * opt_fsize);
 		opt_kbd = (unsigned int)opt_kbd % KBD_MAX;
 		opt_justify = (unsigned int)opt_justify % JUST_MAX;
 		switch (menu_settings_num) {
 		case 0:
 			snprintf(menu_buff, sizeof(menu_buff), SETTINGS_GFX_MENU, 
-			opt_get_mode(), opt_fs?ON:OFF, opt_fsize, just[opt_justify],
+			opt_get_mode(), opt_fs?ON:OFF, opt_hires?ON:OFF, fsize, just[opt_justify],
 				opt_hl?ON:OFF, opt_fading?ON:OFF, opt_owntheme?ON:OFF);
+			if (standalone_sw)
+				menu_strip_tag("<?:owntheme>", "</?>");
+			else
+				menu_remove_tag("<?:owntheme>", "</?>");
 			break;
 		case 1:
 			snprintf(menu_buff, sizeof(menu_buff), SETTINGS_SND_MENU, 
@@ -395,7 +544,11 @@ char *game_menu_gen(void)
 		WARNING_MENU, err_msg?err_msg:UNKNOWN_ERROR);
 		game_err_msg(NULL);
 	} else if (cur_menu == menu_remove) {
-		snprintf(menu_buff, sizeof(menu_buff), REMOVE_MENU, games[gtr].path);
+		const char *sname = games[gtr].path;
+		if (strlen(games[gtr].path) >= 48) {
+			sname = games[gtr].name;
+		}
+		snprintf(menu_buff, sizeof(menu_buff), REMOVE_MENU, sname);
 	}
 	return menu_buff;
 }
@@ -428,19 +581,15 @@ int game_menu_act(const char *a)
 		opt_click ^= 1;
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/mode++")) {
-#ifndef ANDROID
 		if (gfx_next_mode(&opt_mode[0], &opt_mode[1]))
 			opt_mode[0] = opt_mode[1] = -1;
 		restart_needed = 1;
 		game_menu_box(1, game_menu_gen());
-#endif
 	} else if (!strcmp(a, "/mode--")) {	
-#ifndef ANDROID
 		if (gfx_prev_mode(&opt_mode[0], &opt_mode[1]))
 			opt_mode[0] = opt_mode[1] = -1;
 		restart_needed = 1;
 		game_menu_box(1, game_menu_gen());
-#endif
 	} else if (!strcmp(a, "/just++")) {
 		restart_needed = 1;
 		opt_justify ++;
@@ -455,14 +604,14 @@ int game_menu_act(const char *a)
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/fs--")) {
 		opt_fsize --;
-		if (FONT_SZ(game_theme.font_size) > FONT_MIN_SZ * game_theme.scale) {
+		if (opt_fsize >= FONT_MIN_SZ) {
 			restart_needed = 1;
 		} else
 			opt_fsize ++;
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/fs++")) {
 		opt_fsize ++;
-		if (FONT_SZ(game_theme.font_size) < FONT_MAX_SZ * game_theme.scale) {
+		if (opt_fsize <= FONT_MAX_SZ) {
 			restart_needed = 1;
 		} else
 			opt_fsize --;
@@ -474,11 +623,16 @@ int game_menu_act(const char *a)
 		opt_fading ^= 1;
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/fs")) {
-#ifndef ANDROID
+#if !defined(ANDROID) && !defined(IOS)
 		restart_needed = 1;
 		opt_fs ^= 1;
 		game_menu_box(1, game_menu_gen());
 #endif
+	} else if (!strcmp(a, "/hires")) {
+		opt_hires ^= 1;
+		if (opt_fs)
+			restart_needed = 1;
+		game_menu_box(1, game_menu_gen());
 	} else if (!strncmp(a, "/games ", 7)) {
 		if (!strcmp(a + 7, "prev")) {
 			games_menu_from -= MENU_GAMES_MAX;
@@ -494,15 +648,18 @@ int game_menu_act(const char *a)
 		game_menu_box(1, game_menu_gen());
 	} else if (!strncmp(a, "/themes ", 8)) {
 		if (!strcmp(a + 8, "prev")) {
-			themes_menu_from -= MENU_THEMES_MAX;
-			if (themes_menu_from < 0)
-				themes_menu_from = 0;
+			themes_menu_page --;
+			if (themes_menu_page < 0)
+				themes_menu_page = 0;
 		} else if (!strcmp(a + 8, "next")) {
-			if (themes_menu_from + MENU_THEMES_MAX < themes_nr)
-				themes_menu_from += MENU_THEMES_MAX;
+			int count = themes_max(NULL); int pages = 0;
+			if (count > 0)
+				pages = (count - 1) / MENU_THEMES_MAX + 1;
+			if (themes_menu_page + 1 < pages)
+				themes_menu_page ++;
 		} else {
 			int nr = atoi(a + 8);
-			themes_menu_from = nr * MENU_THEMES_MAX;
+			themes_menu_page = nr;
 		}
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/select")) {
@@ -538,18 +695,18 @@ int game_menu_act(const char *a)
 		int nr = atoi(a + 5);
 		if (!curgame_dir)
 			return 0;
-//		free_last();
+/*		free_last(); */
 		game_menu_box(0, NULL);
 		if (!game_reset()) {
 			game_load(nr);
 			cur_menu = menu_main;
 		}
-//		game_menu_box(0, NULL);
+/*		game_menu_box(0, NULL); */
 	} else if (!strcmp(a, "/new")) {
 		char *s;
 		if (!curgame_dir)
 			return 0;
-//		free_last();
+/*		free_last(); */
 
 /* remove autlosave */
 		s = game_save_path(0, 0);
@@ -561,11 +718,15 @@ int game_menu_act(const char *a)
 			custom_theme_warn();
 		}
 	} else if (!strcmp(a,"/main")) {
+		if (top_menu != menu_main) {
+			cur_menu = menu_main;
+			game_menu_box(0, NULL);
+		} else
+			game_menu(menu_main);
 		if (restart_needed) {
 			game_restart();
 			restart_needed = 0;
 		}
-		game_menu(menu_main);
 	} else if (!strcmp(a,"/ask_quit")) {
 		game_menu(menu_askquit);
 	} else if (!strncmp(a, "/remove_", 8)) {
@@ -573,6 +734,8 @@ int game_menu_act(const char *a)
 		game_menu(menu_remove);
 	} else if (!strcmp(a,"/about")) {
 		game_menu(menu_about);
+	} else if (!strcmp(a,"/about-instead")) {
+		game_menu(menu_about_instead);
 	} else if (!strcmp(a,"/mtoggle")) {
 		if (!old_vol) {
 			old_vol = snd_volume_mus(-1);
@@ -617,14 +780,14 @@ int game_menu_act(const char *a)
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/hz-")) {
 		int hz = snd_hz();
-		if (hz == 48000)
-			hz = 44100;
-		else if (hz == 44100)
+		if (hz == 44100)
 			hz = 22050;
 		else if (hz == 22050)
 			hz = 11025;
-		else
+		else if (hz == 11025)
 			hz = 0;
+		else
+			hz = 22050;
 		game_change_hz(hz);
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/hz+")) {
@@ -634,9 +797,9 @@ int game_menu_act(const char *a)
 		else if (hz == 22050)
 			hz = 44100;
 		else if (hz == 44100)
-			hz = 48000;	
-		else
 			hz = 0;
+		else
+			hz = 22050;
 		game_change_hz(hz);
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/lang++")) {
@@ -649,6 +812,7 @@ int game_menu_act(const char *a)
 			instead_lang();
 		themes_rename();
 		games_rename();
+		game_reset_name();
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a, "/lang--")) {
 		do {
@@ -656,7 +820,11 @@ int game_menu_act(const char *a)
 			if (cur_lang < 0)
 			cur_lang = langs_nr - 1;
 		} while (menu_lang_select(langs[cur_lang].file));
+		if (curgame_dir)
+			instead_lang();
 		themes_rename();
+		games_rename();
+		game_reset_name();
 		game_menu_box(1, game_menu_gen());
 	} else if (!strcmp(a,"/quit")) {
 		return -1;
@@ -686,7 +854,7 @@ int game_menu_act(const char *a)
 			game_done(0);
 			if (game_init(og)) {
 				game_error();
-			} else if (curgame_dir && game_own_theme && opt_owntheme) {
+			} else if (curgame_dir && game_own_theme && opt_owntheme && !curtheme_dir[THEME_GAME]) {
 				game_menu(menu_own_theme);
 			}
 			free(p);
@@ -697,12 +865,9 @@ int game_menu_act(const char *a)
 
 void custom_theme_warn(void)
 {
-#ifndef ANDROID
 	if (game_own_theme && !opt_owntheme && cur_menu != menu_warning) {
-		mouse_reset(1);
 		game_menu(menu_custom_theme);
 	}
-#endif
 }
 
 
@@ -762,40 +927,40 @@ static int lang_ok(void)
 }	
 
 struct parser lang_parser[] = {
-	{ "UNKNOWN_ERROR", parse_esc_string, &UNKNOWN_ERROR },
-	{ "ERROR_MENU", parse_esc_string, &ERROR_MENU },
-	{ "WARNING_MENU", parse_esc_string, &WARNING_MENU },
-	{ "SAVE_SLOT_EMPTY", parse_esc_string, &SAVE_SLOT_EMPTY },
-	{ "SELECT_LOAD_MENU", parse_esc_string, &SELECT_LOAD_MENU },
-	{ "AUTOSAVE_SLOT", parse_esc_string, &AUTOSAVE_SLOT },
-	{ "BROKEN_SLOT", parse_esc_string, &BROKEN_SLOT },
-	{ "SELECT_SAVE_MENU", parse_esc_string, &SELECT_SAVE_MENU },
-	{ "MAIN_MENU", parse_esc_string, &MAIN_MENU },
-	{ "ABOUT_MENU", parse_esc_string, &ABOUT_MENU },
-	{ "BACK_MENU", parse_esc_string, &BACK_MENU },
-	{ "SETTINGS_GFX_MENU", parse_esc_string, &SETTINGS_GFX_MENU },
-	{ "SETTINGS_SND_MENU", parse_esc_string, &SETTINGS_SND_MENU },
-	{ "SETTINGS_OTH_MENU", parse_esc_string, &SETTINGS_OTH_MENU },
-	{ "CUSTOM_THEME_MENU", parse_esc_string, &CUSTOM_THEME_MENU },
-	{ "OWN_THEME_MENU", parse_esc_string, &OWN_THEME_MENU },
-	{ "WAIT_MENU", parse_esc_string, &WAIT_MENU },
-	{ "SELECT_GAME_MENU", parse_esc_string, &SELECT_GAME_MENU },
-	{ "SELECT_THEME_MENU", parse_esc_string, &SELECT_THEME_MENU },
-	{ "SAVED_MENU", parse_esc_string, &SAVED_MENU },
-	{ "NOGAMES_MENU", parse_esc_string, &NOGAMES_MENU },
-	{ "NOTHEMES_MENU", parse_esc_string, &NOTHEMES_MENU },
-	{ "QUIT_MENU", parse_esc_string, &QUIT_MENU },
-	{ "REMOVE_MENU", parse_esc_string, &REMOVE_MENU },
-	{ "ON", parse_esc_string, &ON },
-	{ "OFF", parse_esc_string, &OFF },
-	{ "KBD_MODE_LINKS", parse_esc_string, &KBD_MODE_LINKS },
-	{ "KBD_MODE_SMART", parse_esc_string, &KBD_MODE_SMART },
-	{ "KBD_MODE_SCROLL", parse_esc_string, &KBD_MODE_SCROLL },
-	{ "CANCEL_MENU", parse_esc_string, &CANCEL_MENU },
-	{ "FROM_THEME", parse_esc_string, &FROM_THEME },
-	{ "DISABLED_SAVE_MENU", parse_esc_string, &DISABLED_SAVE_MENU },
-	{ "BROWSE_MENU", parse_esc_string, &BROWSE_MENU },
-	{ NULL,  },
+	{ "UNKNOWN_ERROR", parse_esc_string, &UNKNOWN_ERROR, 0 },
+	{ "ERROR_MENU", parse_esc_string, &ERROR_MENU, 0 },
+	{ "WARNING_MENU", parse_esc_string, &WARNING_MENU, 0 },
+	{ "SAVE_SLOT_EMPTY", parse_esc_string, &SAVE_SLOT_EMPTY, 0 },
+	{ "SELECT_LOAD_MENU", parse_esc_string, &SELECT_LOAD_MENU, 0 },
+	{ "AUTOSAVE_SLOT", parse_esc_string, &AUTOSAVE_SLOT, 0 },
+	{ "BROKEN_SLOT", parse_esc_string, &BROKEN_SLOT, 0 },
+	{ "SELECT_SAVE_MENU", parse_esc_string, &SELECT_SAVE_MENU, 0 },
+	{ "MAIN_MENU", parse_esc_string, &MAIN_MENU, 0 },
+	{ "ABOUT_MENU", parse_esc_string, &ABOUT_MENU, 0 },
+	{ "BACK_MENU", parse_esc_string, &BACK_MENU, 0 },
+	{ "SETTINGS_GFX_MENU", parse_esc_string, &SETTINGS_GFX_MENU, 0 },
+	{ "SETTINGS_SND_MENU", parse_esc_string, &SETTINGS_SND_MENU, 0 },
+	{ "SETTINGS_OTH_MENU", parse_esc_string, &SETTINGS_OTH_MENU, 0 },
+	{ "CUSTOM_THEME_MENU", parse_esc_string, &CUSTOM_THEME_MENU, 0 },
+	{ "OWN_THEME_MENU", parse_esc_string, &OWN_THEME_MENU, 0 },
+	{ "WAIT_MENU", parse_esc_string, &WAIT_MENU, 0 },
+	{ "SELECT_GAME_MENU", parse_esc_string, &SELECT_GAME_MENU, 0 },
+	{ "SELECT_THEME_MENU", parse_esc_string, &SELECT_THEME_MENU, 0 },
+	{ "SAVED_MENU", parse_esc_string, &SAVED_MENU, 0 },
+	{ "NOGAMES_MENU", parse_esc_string, &NOGAMES_MENU, 0 },
+	{ "NOTHEMES_MENU", parse_esc_string, &NOTHEMES_MENU, 0 },
+	{ "QUIT_MENU", parse_esc_string, &QUIT_MENU, 0 },
+	{ "REMOVE_MENU", parse_esc_string, &REMOVE_MENU, 0 },
+	{ "ON", parse_esc_string, &ON, 0 },
+	{ "OFF", parse_esc_string, &OFF, 0 },
+	{ "KBD_MODE_LINKS", parse_esc_string, &KBD_MODE_LINKS, 0 },
+	{ "KBD_MODE_SMART", parse_esc_string, &KBD_MODE_SMART, 0 },
+	{ "KBD_MODE_SCROLL", parse_esc_string, &KBD_MODE_SCROLL, 0 },
+	{ "CANCEL_MENU", parse_esc_string, &CANCEL_MENU, 0 },
+	{ "FROM_THEME", parse_esc_string, &FROM_THEME, 0 },
+	{ "DISABLED_SAVE_MENU", parse_esc_string, &DISABLED_SAVE_MENU, 0 },
+	{ "BROWSE_MENU", parse_esc_string, &BROWSE_MENU, 0 },
+	{ NULL,  NULL, NULL, 0 },
 };
 
 static int lang_parse(const char *path)
@@ -855,6 +1020,7 @@ int menu_langs_lookup(const char *path)
 	int n = 0, i = 0;
 	DIR *d;
 	struct dirent *de;
+	struct lang *new_langs;
 
 	if (!path)
 		return 0;
@@ -867,12 +1033,17 @@ int menu_langs_lookup(const char *path)
 			continue;
 		n ++;
 	}
-		
-	rewinddir(d);
 	if (!n)
 		goto out;
-
-	langs = realloc(langs, sizeof(struct lang) * (n + langs_nr));
+	closedir(d); d = opendir(path);
+	if (!d)
+		return -1;
+	new_langs = realloc(langs, sizeof(struct lang) * (n + langs_nr));
+	if (!new_langs) {
+		closedir(d);
+		return -1;
+	}
+	langs = new_langs;
 
 	while ((de = readdir(d)) && i < n) {
 		if (!is_lang(path, de->d_name))
@@ -884,7 +1055,7 @@ int menu_langs_lookup(const char *path)
 		langs_nr ++;
 		i ++;
 	}
-out:	
+out:
 	langs_sort();
 	closedir(d);
 	return 0;
