@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include "externals.h"
 #include "internals.h"
 
@@ -28,6 +52,11 @@ struct _mus_t {
 	SDL_RWops *rw;
 };
 
+int snd_enabled(void)
+{
+	return sound_on;
+}
+
 static void mus_callback(void *aux)
 {
 	if (!timer_id)
@@ -60,7 +89,6 @@ int snd_hz(void)
 	return freq;
 }
 
-int alsa_sw = 0;
 int nosound_sw = 0;
 void snd_pause(int on)
 {
@@ -94,11 +122,7 @@ int snd_init(int hz)
 		fprintf(stderr, "Unable to init audio!\n");
 		return -1;
 	}
-	
-	if (alsa_sw) {
-		SDL_AudioInit("alsa");
-	}
-	
+
 	if (Mix_OpenAudio(hz, audio_format, audio_channels, audio_buffers)) {
 		fprintf(stderr, "Unable to open audio!\n");
 		return -1;
@@ -128,7 +152,6 @@ wav_t	snd_load_wav(const char *fname)
 		return NULL;
 	rw = RWFromIdf(game_idf, fname);
 	if (!rw || !(r = (wav_t)Mix_LoadWAV_RW(rw, 1))) {
-		fprintf(stderr,"Can't load '%s'.\n", fname);
 		return NULL;
 	}
 	return r;
@@ -138,7 +161,7 @@ void	snd_free_wav(wav_t w)
 {	
 	if (!w)
 		return;	
-//	Mix_HaltChannel(-1);
+/*	Mix_HaltChannel(-1); */
 	Mix_FreeChunk((Mix_Chunk*)w);
 }
 void snd_halt_chan(int han, int ms)
@@ -199,10 +222,8 @@ int snd_play_mus(char *fname, int ms, int loop)
 		snd_free_mus(mus);
 
 	mus = snd_load_mus(fname);
-	if (!mus) {
-		fprintf(stderr,"Can't load '%s'.\n", fname);
-		return 0;
-	}
+	if (!mus)
+		return -1;
 	if (loop >= 0)
 		Mix_HookMusicFinished(game_music_finished);
 	else
@@ -211,7 +232,7 @@ int snd_play_mus(char *fname, int ms, int loop)
 		Mix_FadeInMusic(mus->mus, loop, ms);
 	else
 		Mix_PlayMusic(mus->mus, loop);
-	snd_volume_mus(snd_volume_mus(-1)); // SDL hack?
+	snd_volume_mus(snd_volume_mus(-1)); /* SDL hack? */
 	return 0;
 }
 
@@ -291,7 +312,7 @@ int snd_play(void *chunk, int channel, int loop)
 		channel = -1;
 	if (channel != -1)
 		snd_halt_chan(channel, 0);
-	snd_volume_mus(snd_volume_mus(-1)); // SDL hack?
+	snd_volume_mus(snd_volume_mus(-1)); /* SDL hack? */
 	return Mix_PlayChannel(channel, (Mix_Chunk*)chunk, loop);
 }
 
