@@ -1,9 +1,34 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include "externals.h"
 #include "internals.h"
 
 int theme_relative = 0;
 
-char	*curtheme_dir = NULL;
+char	*curtheme_dir[2] = { NULL, NULL };
+struct theme *curtheme_loading = NULL;
 
 static int parse_win_align(const char *v, void *data)
 {
@@ -123,7 +148,34 @@ static int parse_inv_mode(const char *v, void *data)
 		*i = INV_MODE_VERT | INV_ALIGN_SET(ALIGN_CENTER);
 	else
 		return -1;
-	return 0;	
+	return 0;
+}
+
+static int parse_ways_mode(const char *v, void *data)
+{
+	int *i = (int *)data;
+	if (!strcmp(v, "top"))
+		*i = ALIGN_TOP;
+	else if (!strcmp(v, "bottom"))
+		*i = ALIGN_BOTTOM;
+	else
+		return -1;
+	return 0;
+}
+
+static int out_ways_mode(const void *v, char **out)
+{
+	char *o;
+	int m = *((int*)v);
+	o = malloc(64);
+	if (!o)
+		return -1;
+	if (m == ALIGN_BOTTOM)
+		sprintf(o, "bottom");
+	else
+		sprintf(o, "top");
+	*out = o;
+	return 0;
 }
 
 static int out_inv_mode(const void *v, char **out)
@@ -179,95 +231,126 @@ static int parse_include(const char *v, void *data)
 	char cwd[PATH_MAX];
 	if (!strlowcmp(v, DEFAULT_THEME))
 		return 0;
+	if (curtheme_loading && strlowcmp(v, curtheme_loading->dir) &&
+		theme_lookup(v, curtheme_loading->type)) /* internal theme? */
+		return game_theme_load(v, THEME_GAME);
 	getdir(cwd, sizeof(cwd));
 	setdir(game_cwd);
-	rc = game_theme_load(v);
-//	if (!rc)
-//		game_theme_select(v);
+	rc = game_theme_load(v, THEME_GLOBAL);
+/*	if (!rc)
+		game_theme_select(v); */
 	setdir(cwd);
 	return rc;
 }
+
+static int theme_parse_full_path(const char *v, void *data)
+{
+	int rc;
+	char **p = (char **)data;
+	char *np;
+	if (theme_relative) {
+		if (!strncmp(v, "blank:", 6) || 
+			!strncmp(v, "box:", 4) ||
+			!strncmp(v, "spr:", 4)) /* hack for special files*/
+			return parse_path(v, data);
+		rc = parse_path(v, data);
+		if (rc || !*p || !*p[0])
+			return rc;
+
+		if (curtheme_loading && curtheme_loading->type == THEME_GAME) {
+			np = getfilepath(curtheme_loading->path, *p);
+			if (!*np)
+				return -1;
+			free(*p); *p = np;
+		}
+		return 0;
+	}
+	return parse_full_path(v, data);
+}
+
 struct parser cmd_parser[] = {
-	{ "scr.w", parse_int, &game_theme.w },
-	{ "scr.h", parse_int, &game_theme.h },
-	{ "scr.gfx.scalable", parse_int, &game_theme.gfx_scalable },
-	{ "scr.col.bg", parse_color, &game_theme.bgcol },
-	{ "scr.gfx.bg", parse_full_path, &game_theme.bg_name, CHANGED_BG },
-	{ "scr.gfx.cursor.normal", parse_full_path, &game_theme.cursor_name, CHANGED_CURSOR },
-	{ "scr.gfx.cursor.x", parse_int, &game_theme.cur_x },
-	{ "scr.gfx.cursor.y", parse_int, &game_theme.cur_y },
-	{ "scr.gfx.use", parse_full_path, &game_theme.use_name, CHANGED_USE }, /* compat */
-	{ "scr.gfx.cursor.use", parse_full_path, &game_theme.use_name, CHANGED_USE },
-	{ "scr.gfx.pad", parse_int, &game_theme.pad }, 
-	{ "scr.gfx.x", parse_int, &game_theme.gfx_x },
-	{ "scr.gfx.y", parse_int, &game_theme.gfx_y },
-	{ "scr.gfx.w", parse_int, &game_theme.max_scene_w },
-	{ "scr.gfx.h", parse_int, &game_theme.max_scene_h },
-	{ "scr.gfx.mode", parse_gfx_mode, &game_theme.gfx_mode },
+	{ "scr.w", parse_int, &game_theme.w, 0 },
+	{ "scr.h", parse_int, &game_theme.h, 0 },
+	{ "scr.gfx.scalable", parse_int, &game_theme.gfx_scalable, 0 },
+	{ "scr.col.bg", parse_color, &game_theme.bgcol, 0 },
+	{ "scr.gfx.icon", theme_parse_full_path, &game_theme.icon_name, CHANGED_ICON },
+	{ "scr.gfx.bg", theme_parse_full_path, &game_theme.bg_name, CHANGED_BG },
+	{ "scr.gfx.cursor.normal", theme_parse_full_path, &game_theme.cursor_name, CHANGED_CURSOR },
+	{ "scr.gfx.cursor.x", parse_int, &game_theme.cur_x, 0 },
+	{ "scr.gfx.cursor.y", parse_int, &game_theme.cur_y, 0 },
+	{ "scr.gfx.use", theme_parse_full_path, &game_theme.use_name, CHANGED_USE }, /* compat */
+	{ "scr.gfx.cursor.use", theme_parse_full_path, &game_theme.use_name, CHANGED_USE },
+	{ "scr.gfx.pad", parse_int, &game_theme.pad, 0 }, 
+	{ "scr.gfx.x", parse_int, &game_theme.gfx_x, 0 },
+	{ "scr.gfx.y", parse_int, &game_theme.gfx_y, 0 },
+	{ "scr.gfx.w", parse_int, &game_theme.max_scene_w, 0 },
+	{ "scr.gfx.h", parse_int, &game_theme.max_scene_h, 0 },
+	{ "scr.gfx.mode", parse_gfx_mode, &game_theme.gfx_mode, 0 },
 
-	{ "win.align", parse_win_align, &game_theme.win_align },
-	{ "win.x", parse_int, &game_theme.win_x },
-	{ "win.y", parse_int, &game_theme.win_y },
-	{ "win.w", parse_int, &game_theme.win_w },
-	{ "win.h", parse_int, &game_theme.win_h },
-	{ "win.scroll.mode", parse_int, &game_theme.win_scroll_mode },
+	{ "win.align", parse_win_align, &game_theme.win_align, 0 },
+	{ "win.x", parse_int, &game_theme.win_x, 0 },
+	{ "win.y", parse_int, &game_theme.win_y, 0 },
+	{ "win.w", parse_int, &game_theme.win_w, 0 },
+	{ "win.h", parse_int, &game_theme.win_h, 0 },
+	{ "win.ways.mode", parse_ways_mode, &game_theme.ways_mode, 0 },
+	{ "win.scroll.mode", parse_int, &game_theme.win_scroll_mode, 0 },
 
-	{ "win.fnt.name", parse_full_path, &game_theme.font_name, CHANGED_FONT },
+	{ "win.fnt.name", theme_parse_full_path, &game_theme.font_name, CHANGED_FONT },
 	{ "win.fnt.size", parse_int, &game_theme.font_size, CHANGED_FONT },
-	{ "win.fnt.height", parse_float, &game_theme.font_height },
+	{ "win.fnt.height", parse_float, &game_theme.font_height, 0 },
 /* compat mode directive */
-	{ "win.gfx.h", parse_int, &game_theme.max_scene_h },
+	{ "win.gfx.h", parse_int, &game_theme.max_scene_h, 0 },
 /* here it was */
-	{ "win.gfx.up", parse_full_path, &game_theme.a_up_name, CHANGED_UP },
-	{ "win.gfx.down", parse_full_path, &game_theme.a_down_name, CHANGED_DOWN },
-	{ "win.up.x", parse_int, &game_theme.a_up_x },
-	{ "win.up.y", parse_int, &game_theme.a_up_y },
-	{ "win.down.x", parse_int, &game_theme.a_down_x },
-	{ "win.down.y", parse_int, &game_theme.a_down_y },
-	{ "win.col.fg", parse_color, &game_theme.fgcol }, 
-	{ "win.col.link", parse_color, &game_theme.lcol },
-	{ "win.col.alink", parse_color, &game_theme.acol }, 
+	{ "win.gfx.up", theme_parse_full_path, &game_theme.a_up_name, CHANGED_UP },
+	{ "win.gfx.down", theme_parse_full_path, &game_theme.a_down_name, CHANGED_DOWN },
+	{ "win.up.x", parse_int, &game_theme.a_up_x, 0 },
+	{ "win.up.y", parse_int, &game_theme.a_up_y, 0 },
+	{ "win.down.x", parse_int, &game_theme.a_down_x, 0 },
+	{ "win.down.y", parse_int, &game_theme.a_down_y, 0 },
+	{ "win.col.fg", parse_color, &game_theme.fgcol, 0 }, 
+	{ "win.col.link", parse_color, &game_theme.lcol, 0 },
+	{ "win.col.alink", parse_color, &game_theme.acol, 0 }, 
 
-	{ "inv.x", parse_int, &game_theme.inv_x },
-	{ "inv.y", parse_int, &game_theme.inv_y },
-	{ "inv.w", parse_int, &game_theme.inv_w },
-	{ "inv.h", parse_int, &game_theme.inv_h },
-	{ "inv.mode", parse_inv_mode, &game_theme.inv_mode },
-	{ "inv.horiz", parse_inv_mode, &game_theme.inv_mode },
+	{ "inv.x", parse_int, &game_theme.inv_x, 0 },
+	{ "inv.y", parse_int, &game_theme.inv_y, 0 },
+	{ "inv.w", parse_int, &game_theme.inv_w, 0 },
+	{ "inv.h", parse_int, &game_theme.inv_h, 0 },
+	{ "inv.mode", parse_inv_mode, &game_theme.inv_mode, 0 },
+	{ "inv.horiz", parse_inv_mode, &game_theme.inv_mode, 0 },
 
-	{ "inv.col.fg", parse_color, &game_theme.icol },
-	{ "inv.col.link", parse_color, &game_theme.ilcol },
-	{ "inv.col.alink", parse_color, &game_theme.iacol }, 
-	{ "inv.fnt.name", parse_full_path, &game_theme.inv_font_name, CHANGED_IFONT },
+	{ "inv.col.fg", parse_color, &game_theme.icol, 0 },
+	{ "inv.col.link", parse_color, &game_theme.ilcol, 0 },
+	{ "inv.col.alink", parse_color, &game_theme.iacol, 0 }, 
+	{ "inv.fnt.name", theme_parse_full_path, &game_theme.inv_font_name, CHANGED_IFONT },
 	{ "inv.fnt.size", parse_int, &game_theme.inv_font_size, CHANGED_IFONT },
-	{ "inv.fnt.height", parse_float, &game_theme.inv_font_height },
-	{ "inv.gfx.up", parse_full_path, &game_theme.inv_a_up_name, CHANGED_IUP },
-	{ "inv.gfx.down", parse_full_path, &game_theme.inv_a_down_name, CHANGED_IDOWN },
-	{ "inv.up.x", parse_int, &game_theme.inv_a_up_x },
-	{ "inv.up.y", parse_int, &game_theme.inv_a_up_y },
-	{ "inv.down.x", parse_int, &game_theme.inv_a_down_x },
-	{ "inv.down.y", parse_int, &game_theme.inv_a_down_y },
+	{ "inv.fnt.height", parse_float, &game_theme.inv_font_height, 0 },
+	{ "inv.gfx.up", theme_parse_full_path, &game_theme.inv_a_up_name, CHANGED_IUP },
+	{ "inv.gfx.down", theme_parse_full_path, &game_theme.inv_a_down_name, CHANGED_IDOWN },
+	{ "inv.up.x", parse_int, &game_theme.inv_a_up_x, 0 },
+	{ "inv.up.y", parse_int, &game_theme.inv_a_up_y, 0 },
+	{ "inv.down.x", parse_int, &game_theme.inv_a_down_x, 0 },
+	{ "inv.down.y", parse_int, &game_theme.inv_a_down_y, 0 },
 
-	{ "menu.col.bg", parse_color, &game_theme.menu_bg },
-	{ "menu.col.fg", parse_color, &game_theme.menu_fg },
-	{ "menu.col.link", parse_color, &game_theme.menu_link },
-	{ "menu.col.alink", parse_color, &game_theme.menu_alink },
-	{ "menu.col.alpha", parse_int, &game_theme.menu_alpha },
-	{ "menu.col.border", parse_color, &game_theme.border_col },
-	{ "menu.bw", parse_int, &game_theme.border_w},
-	{ "menu.fnt.name", parse_full_path, &game_theme.menu_font_name, CHANGED_MFONT },
+	{ "menu.col.bg", parse_color, &game_theme.menu_bg, 0 },
+	{ "menu.col.fg", parse_color, &game_theme.menu_fg, 0 },
+	{ "menu.col.link", parse_color, &game_theme.menu_link, 0 },
+	{ "menu.col.alink", parse_color, &game_theme.menu_alink, 0 },
+	{ "menu.col.alpha", parse_int, &game_theme.menu_alpha, 0 },
+	{ "menu.col.border", parse_color, &game_theme.border_col, 0 },
+	{ "menu.bw", parse_int, &game_theme.border_w, 0 },
+	{ "menu.fnt.name", theme_parse_full_path, &game_theme.menu_font_name, CHANGED_MFONT },
 	{ "menu.fnt.size", parse_int, &game_theme.menu_font_size, CHANGED_MFONT },
-	{ "menu.fnt.height", parse_float, &game_theme.menu_font_height },
-	{ "menu.gfx.button", parse_full_path, &game_theme.menu_button_name, CHANGED_BUTTON },
-	{ "menu.button.x", parse_int, &game_theme.menu_button_x },
-	{ "menu.button.y", parse_int, &game_theme.menu_button_y },
+	{ "menu.fnt.height", parse_float, &game_theme.menu_font_height, 0 },
+	{ "menu.gfx.button", theme_parse_full_path, &game_theme.menu_button_name, CHANGED_BUTTON },
+	{ "menu.button.x", parse_int, &game_theme.menu_button_x, 0 },
+	{ "menu.button.y", parse_int, &game_theme.menu_button_y, 0 },
 /* compat */
-	{ "menu.buttonx", parse_int, &game_theme.menu_button_x },
-	{ "menu.buttony", parse_int, &game_theme.menu_button_y },
+	{ "menu.buttonx", parse_int, &game_theme.menu_button_x, 0 },
+	{ "menu.buttony", parse_int, &game_theme.menu_button_y, 0 },
 
-	{ "snd.click", parse_full_path, &game_theme.click_name, CHANGED_CLICK },
-	{ "include", parse_include, NULL },
-	{ NULL,  },
+	{ "snd.click", theme_parse_full_path, &game_theme.click_name, CHANGED_CLICK },
+	{ "include", parse_include, NULL, 0 },
+	{ NULL, NULL, NULL, 0 },
 };
 
 #define TF_POSX	1
@@ -281,26 +364,26 @@ typedef struct {
 } theme_scalable_t;
 
 static theme_scalable_t theme_scalables[] = {
-	{ "scr.w", &game_theme.w },
-	{ "scr.h", &game_theme.h },
-	{ "scr.gfx.cursor.x", &game_theme.cur_x },
-	{ "scr.gfx.cursor.y", &game_theme.cur_y },
-	{ "scr.gfx.pad", &game_theme.pad }, 
+	{ "scr.w", &game_theme.w, 0 },
+	{ "scr.h", &game_theme.h, 0 },
+	{ "scr.gfx.cursor.x", &game_theme.cur_x, 0 },
+	{ "scr.gfx.cursor.y", &game_theme.cur_y, 0 },
+	{ "scr.gfx.pad", &game_theme.pad, 0 }, 
 	{ "scr.gfx.x", &game_theme.gfx_x, TF_POSX },
 	{ "scr.gfx.y", &game_theme.gfx_y, TF_POSY },
 	{ "scr.gfx.w", &game_theme.max_scene_w, TF_NEG },
 	{ "scr.gfx.h", &game_theme.max_scene_h, TF_NEG },
 	{ "win.x", &game_theme.win_x, TF_POSX },
 	{ "win.y", &game_theme.win_y, TF_POSY },
-	{ "win.w", &game_theme.win_w },
-	{ "win.h", &game_theme.win_h },
-	{ "win.fnt.size", &game_theme.font_size },
+	{ "win.w", &game_theme.win_w, 0 },
+	{ "win.h", &game_theme.win_h, 0 },
+	{ "win.fnt.size", &game_theme.font_size, 0 },
 	{ "inv.x", &game_theme.inv_x, TF_POSX },
 	{ "inv.y", &game_theme.inv_y, TF_POSY },
-	{ "inv.w", &game_theme.inv_w },
-	{ "inv.h", &game_theme.inv_h },
-	{ "inv.fnt.size", &game_theme.inv_font_size },
-	{ "menu.fnt.size", &game_theme.menu_font_size },
+	{ "inv.w", &game_theme.inv_w, 0 },
+	{ "inv.h", &game_theme.inv_h, 0 },
+	{ "inv.fnt.size", &game_theme.inv_font_size, 0 },
+	{ "menu.fnt.size", &game_theme.menu_font_size, 0 },
 	{ "menu.button.x", &game_theme.menu_button_x, TF_POSX },
 	{ "menu.button.y", &game_theme.menu_button_y, TF_POSY },
 	{ "win.up.x", &game_theme.a_up_x, TF_POSX | TF_NEG },
@@ -311,7 +394,7 @@ static theme_scalable_t theme_scalables[] = {
 	{ "inv.up.y", &game_theme.inv_a_up_y, TF_POSY | TF_NEG },
 	{ "inv.down.x", &game_theme.inv_a_down_x, TF_POSX | TF_NEG },
 	{ "inv.down.y", &game_theme.inv_a_down_y, TF_POSY | TF_NEG },
-	{ NULL,  },
+	{ NULL, NULL, 0 },
 };
 static int theme_scalables_unscaled[sizeof(theme_scalables)/sizeof(theme_scalable_t)];
 
@@ -324,6 +407,8 @@ struct game_theme game_theme = {
 	.bg = NULL,
 	.use_name = NULL,
 	.cursor_name = NULL,
+	.icon = NULL,
+	.icon_name = NULL,
 	.use = NULL,
 	.cursor = NULL,
 	.cur_x = 0,
@@ -359,6 +444,7 @@ struct game_theme game_theme = {
 	.win_scroll_mode = 2,
 	.gfx_mode = GFX_MODE_EMBEDDED,
 	.inv_mode = INV_MODE_VERT | INV_ALIGN_SET(ALIGN_LEFT),
+	.ways_mode = ALIGN_TOP,
 	.click_name = NULL,
 	.click = NULL,
 	.xoff = 0,
@@ -371,6 +457,7 @@ static void free_theme_strings(void)
 {
 	struct game_theme *t = &game_theme;
 	FREE(t->use_name);
+	FREE(t->icon_name);
 	FREE(t->cursor_name);
 	FREE(t->bg_name);
 	FREE(t->inv_a_up_name);
@@ -419,6 +506,12 @@ int game_theme_free(void)
 	if (game_theme.click) {
 		sound_put(game_theme.click);
 	}
+
+	if (game_theme.icon) {
+		gfx_set_icon(NULL);
+		gfx_free_image(game_theme.icon);
+	}
+
 	game_theme.font = game_theme.inv_font = game_theme.menu_font = NULL;
 	game_theme.a_up = game_theme.a_down = game_theme.use = NULL;
 	game_theme.inv_a_up = game_theme.inv_a_down = NULL;
@@ -427,6 +520,7 @@ int game_theme_free(void)
 	game_theme.click = NULL;
 	game_theme.cur_x = game_theme.cur_y = 0;
 	game_theme.cursor = game_theme.use = NULL;
+	game_theme.icon = NULL;
 	return 0;
 }
 
@@ -540,7 +634,7 @@ char *theme_getvar(char *name)
 			num = (int *)cmd_parser[i].p;
 			sprintf(buf, "%d", *num);
 			return strdup(buf);
-		} else if (cmd_parser[i].fn == parse_full_path) {
+		} else if (cmd_parser[i].fn == theme_parse_full_path) {
 			s = *((char **)cmd_parser[i].p);
 			if (!s)
 				return NULL;
@@ -549,6 +643,10 @@ char *theme_getvar(char *name)
 			if (out_inv_mode(cmd_parser[i].p, &s))
 				return NULL;
 			return s;
+		} else if (cmd_parser[i].fn == parse_ways_mode) {
+			if (out_ways_mode(cmd_parser[i].p, &s))
+				return NULL;
+			return s;
 		} else if (cmd_parser[i].fn == parse_gfx_mode) {
 			if (out_gfx_mode(cmd_parser[i].p, &s))
 				return NULL;
@@ -673,17 +771,22 @@ int game_theme_optimize(void)
 static int game_theme_update_data(void)
 {
 	struct game_theme *t = &game_theme;
+	const char *res = NULL;
 	int idf = idf_only(game_idf, 0);
 	if (t->font_name && (t->changed & CHANGED_FONT)) {
 		fnt_free(t->font);
-		if (!(t->font = fnt_load(t->font_name, FONT_SZ(t->font_size))))
+		if (!(t->font = fnt_load(t->font_name, FONT_SZ(t->font_size)))) {
+			res = t->font_name;
 			goto err;
+		}
 	}
-	
+
 	if (t->inv_font_name && (t->changed & CHANGED_IFONT)) {
 		fnt_free(t->inv_font);
-		if (!(t->inv_font = fnt_load(t->inv_font_name, FONT_SZ(t->inv_font_size))))
+		if (!(t->inv_font = fnt_load(t->inv_font_name, FONT_SZ(t->inv_font_size)))) {
+			res = t->inv_font_name;
 			goto err;
+		}
 	}
 
 	if (t->menu_font_name && (t->changed & CHANGED_MFONT)) {
@@ -692,40 +795,50 @@ static int game_theme_update_data(void)
 			m = game_theme.h / MAX_MENU_LINES / game_theme.menu_font_height;
 		else if (m < t->menu_font_size)
 			m = t->menu_font_size;
-//		fprintf(stderr, "%d %d > %d? %d", (int)FONT_SZ(t->inv_font_size), (int)FONT_SZ(t->inv_font_size) * MAX_MENU_LINES, game_theme.h, m);
+/*		fprintf(stderr, "%d %d > %d? %d", (int)FONT_SZ(t->inv_font_size), (int)FONT_SZ(t->inv_font_size) * MAX_MENU_LINES, game_theme.h, m); */
 		fnt_free(t->menu_font);
-		if (!(t->menu_font = fnt_load(t->menu_font_name, m))) /* do not scale menu!!! */
+		if (!(t->menu_font = fnt_load(t->menu_font_name, m))) { /* do not scale menu!!! */
+			res = t->menu_font_name;
 			goto err;
+		}
 	}
 
 	if (t->a_up_name && (t->changed & CHANGED_UP)) {
 		gfx_free_image(t->a_up);
-		if (!(t->a_up = gfx_load_image(t->a_up_name)))
+		if (!(t->a_up = gfx_load_image(t->a_up_name))) {
+			res = t->a_up_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->a_up))
 			goto err;
 	}
-	
+
 	if (t->a_down_name && (t->changed & CHANGED_DOWN)) {
 		gfx_free_image(t->a_down);
-		if (!(t->a_down = gfx_load_image(t->a_down_name)))
+		if (!(t->a_down = gfx_load_image(t->a_down_name))) {
+			res = t->a_down_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->a_down))
 			goto err;
 	}
 
 	if (t->inv_a_up_name && (t->changed & CHANGED_IUP)) {
 		gfx_free_image(t->inv_a_up);
-		if (!(t->inv_a_up = gfx_load_image(t->inv_a_up_name)))
+		if (!(t->inv_a_up = gfx_load_image(t->inv_a_up_name))) {
+			res = t->inv_a_up_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->inv_a_up))
 			goto err;
 	}
 
 	if (t->inv_a_down_name && (t->changed & CHANGED_IDOWN)) {
 		gfx_free_image(t->inv_a_down);
-		if (!(t->inv_a_down = gfx_load_image(t->inv_a_down_name)))
+		if (!(t->inv_a_down = gfx_load_image(t->inv_a_down_name))) {
+			res = t->inv_a_down_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->inv_a_down))
 			goto err;
 	}
@@ -733,26 +846,38 @@ static int game_theme_update_data(void)
 	if (t->bg_name && (t->changed & CHANGED_BG)) {
 		gfx_free_image(t->bg);
 		t->bg = NULL;
-		if (t->bg_name[0] && !(t->bg = gfx_load_image(t->bg_name)))
+		if (t->bg_name[0] && !(t->bg = gfx_load_image(t->bg_name))) {
+			res = t->bg_name;
 			goto skip; /* not fatal */
+		}
 		if (theme_img_scale(&t->bg))
 			goto err;
 		if (theme_bg_scale())
 			goto err;
 	}
 skip:
+	if (t->icon_name && (t->changed & CHANGED_ICON)) {
+		if (t->icon)
+			gfx_free_image(t->icon);
+		t->icon = gfx_load_image(t->icon_name);
+	}
+
 	if (t->use_name && (t->changed & CHANGED_USE)) {
 		gfx_free_image(t->use);	
-		if (!(t->use = gfx_load_image(t->use_name)))
+		if (!(t->use = gfx_load_image(t->use_name))) {
+			res = t->use_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->use))
 			goto err;
 	}
 
 	if (t->cursor_name && (t->changed & CHANGED_CURSOR)) {
 		gfx_free_image(t->cursor);	
-		if (!(t->cursor = gfx_load_image(t->cursor_name)))
+		if (!(t->cursor = gfx_load_image(t->cursor_name))) {
+			res = t->cursor_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->cursor))
 			goto err;
 		gfx_set_cursor(t->cursor, t->cur_x, t->cur_y);
@@ -760,8 +885,10 @@ skip:
 	
 	if (t->menu_button_name && (t->changed & CHANGED_BUTTON)) {
 		gfx_free_image(t->menu_button);
-		if (!(t->menu_button = gfx_load_image(t->menu_button_name)))
+		if (!(t->menu_button = gfx_load_image(t->menu_button_name))) {
+			res = t->menu_button_name;
 			goto err;
+		}
 		if (theme_img_scale(&t->menu_button))
 			goto err;
 	}
@@ -771,7 +898,7 @@ skip:
 		t->click = sound_get(t->click_name);
 	}
 
-//	free_theme_strings(); /* todo, font */
+/*	free_theme_strings(); */ /* todo, font */
 	t->changed = 0;
 	if (!t->cursor || !t->use || !t->inv_a_up || !t->inv_a_down || !t->a_down || !t->a_up ||
 		!t->font || !t->inv_font || !t->menu_font || !t->menu_button) {
@@ -783,6 +910,7 @@ skip:
 err:
 	idf_only(game_idf, idf);
 	t->changed = 0;
+	game_res_err_msg(res, 1);
 	return -1;
 }
 
@@ -803,32 +931,53 @@ int game_theme_update(void)
 	return 0;
 }
 
+#if defined(ANDROID) || defined(IOS)
+extern void rotate_landscape(void);
+extern void rotate_portrait(void);
+extern void unlock_rotation(void);
+#endif
+
 int game_theme_init(void)
 {
-	color_t col = { .r = 0, .g = 0, .b = 0 };
+	color_t col = { .r = 0, .g = 0, .b = 0, .a = 0 };
 	int w  = opt_mode[0];
 	int h  = opt_mode[1];
 
+	game_cursor_show = 1;
+
+	if (opt_fs && opt_hires && !gfx_get_max_mode(&w, &h, MODE_ANY)) {
+#if defined(IOS) || defined(ANDROID)
+		if ((game_theme.w > game_theme.h && w < h) ||
+			(game_theme.w < game_theme.h && w > h)) { /* rotated */
+			if (gfx_get_max_mode(&w, &h, (game_theme.w > game_theme.h)?MODE_H:MODE_V)) {
+				gfx_get_max_mode(&w, &h, MODE_ANY); /* fallback to any mode */
+			}
+		}
+#if defined(ANDROID) || defined(IOS)
+		if (game_theme.w > game_theme.h)
+			rotate_landscape();
+		else if (game_theme.w < game_theme.h)
+			rotate_portrait();
+		else
+			unlock_rotation();
+#endif
+#endif
+	}
+#if defined(ANDROID) || defined(IOS)
+	else {
+		unlock_rotation();
+	}
+#endif
+
 	if (w == -1) { /* as theme */
-		if (gfx_get_max_mode(&w, &h) || (game_theme.w <= w && game_theme.h <= h)) {
+#if !defined(IOS) /* IOS always hardware accelerated */
+		if (gfx_get_max_mode(&w, &h, MODE_ANY) || (game_theme.w <= w && game_theme.h <= h)) {
 			w = opt_mode[0];
 			h = opt_mode[1];
 		}
-	}
-#if 0 /* not useful */
-	 else if (opt_fs) { /* selected and full screen */
-		if (gfx_get_max_mode(&w, &h) || (opt_mode[0] <= w && opt_mode[1] <= h)) {
-			w = opt_mode[0];
-			h = opt_mode[1];
-		}
-	}
 #endif
-#if 0
-	if (!SCALABLE_THEME && (w < game_theme.w || h < game_theme.h)) { /* no scalable? TODO: message? */
-		w = game_theme.w;
-		h = game_theme.h;
 	}
-#endif
+
 	game_theme_scale(w, h);
 
 	if (gfx_set_mode(game_theme.w, game_theme.h, opt_fs)) {
@@ -845,40 +994,51 @@ int game_theme_init(void)
 	return 0;
 }
 
-static int theme_parse(const char *path)
+static int theme_parse_idf(idf_t idf, const char *path)
 {
-	idff_t idf = NULL;
+	idff_t idff = NULL;
+	if (idf)
+		idff = idf_open(idf, path);
 
-	if (theme_relative)
-		idf = idf_open(game_idf, path);
-
-	if (idf) {
-		int rc = parse_idff(idf, path, cmd_parser);
-		idf_close(idf);
+	if (idff) {
+		int rc = parse_idff(idff, path, cmd_parser);
+		idf_close(idff);
 		if (rc)
 			fprintf(stderr, "Theme parsed with errors!\n");
 		return rc;
 	}
-
 	if (parse_ini(dirpath(path), cmd_parser)) {
 		fprintf(stderr, "Theme parsed with errors!\n");
-//		game_theme_free();
+/*		game_theme_free(); */
 		return -1;
 	}
+
 	return 0;
 }
 
 int theme_load(const char *name)
 {
-	if (theme_parse(name))
+	idf_t idf = NULL;
+
+	if (theme_relative)
+		idf = game_idf;
+
+	if (theme_parse_idf(idf, name))
+		return 0; /* no theme loaded if error in parsing */
+	return 0;
+}
+
+int theme_load_idf(idf_t idf, const char *name)
+{
+	if (theme_parse_idf(idf, name))
 		return 0; /* no theme loaded if error in parsing */
 	return 0;
 }
 
 struct	theme *themes = NULL;
-int	themes_nr = 0;
+int themes_nr = 0;
 
-static int is_theme(const char *path, const char *n)
+static int is_theme_idf(idf_t idf, const char *path, const char *n)
 {
 	int rc = 0;
 	char *p = getpath(path, n);
@@ -889,17 +1049,28 @@ static int is_theme(const char *path, const char *n)
 	if (pp) {
 		strcpy(pp, p);
 		strcat(pp, THEME_FILE);
-		if (!access(pp, R_OK))
-			rc = 1;
+		if (idf)
+			rc = !idf_access(idf, pp);
+		else
+			rc = !access(pp, R_OK);
 		free(pp);
 	}
 	free(p);
 	return rc;
 }
 
+static int is_theme(const char *path, const char *n)
+{
+	if (!n)
+		return 0;
+	if (!strcmp("..", n) || !strcmp(".", n))
+		return 0;
+	return is_theme_idf(NULL, path, n);
+}
+
 static char *theme_name(const char *path, const char *d_name)
 {
-	char *l;	
+	char *l;
 	char *p = getfilepath(path, THEME_FILE);
 	if (!p)
 		goto err;
@@ -911,10 +1082,26 @@ err:
 	return strdup(d_name);
 }
 
+static char *theme_name_idf(idf_t idf, const char *path, const char *d_name)
+{
+	char *l;	
+	char *p = getfilepath(path, THEME_FILE);
+	if (!p)
+		goto err;
+	l = lookup_lang_tag_idf(idf, p, "Name", ";");
+	free(p);
+	if (l) 
+		return l;
+err:
+	return strdup(d_name);
+}
+
 static int cmp_theme(const void *p1, const void *p2)
 {
 	const struct theme *t1 = (const struct theme*)p1;
 	const struct theme *t2 = (const struct theme*)p2;
+	if (t1->type != t2->type)
+		return t1->type - t2->type;
 	return strcmp(t1->name, t2->name);
 }
 
@@ -923,14 +1110,122 @@ static void themes_sort()
 	qsort(themes, themes_nr, sizeof(struct theme), cmp_theme);
 }
 
-static struct theme *theme_lookup(const char *name);
+struct theme *theme_lookup(const char *name, int type);
 
-int themes_lookup(const char *path)
+int themes_count(int type)
+{
+	int rc = 0;
+	int i;
+	for (i = 0; i < themes_nr; i++)
+		rc = rc + (themes[i].type == type);
+	return rc;
+}
+
+int themes_max(int *type)
+{
+	int n;
+	int count = themes_count(THEME_GLOBAL);
+
+	if (type)
+		*type = THEME_GLOBAL;
+
+	if ((n = themes_count(THEME_GAME)) > 0 && opt_owntheme) {
+		count = n;
+		if (type)
+			*type = THEME_GAME;
+	}
+
+	return count;
+}
+
+void themes_drop(int type)
+{
+	int new_size;
+	struct theme *new_themes = NULL;
+	int rc, i, k = 0;
+	rc  = themes_count(type);
+	if (!rc)
+		return;
+	new_size = (themes_nr - rc) * sizeof(struct theme);
+	if (new_size)
+		new_themes = malloc(new_size);
+
+	for (i = 0; i < themes_nr; i ++) {
+		if (themes[i].type == type) {
+			free(themes[i].path);
+			free(themes[i].dir);
+			free(themes[i].name);
+		} else {
+			char *p = curtheme_dir[themes[i].type];
+			new_themes[k++] = themes[i];
+			if (p && !strlowcmp(p, themes[i].dir))
+				curtheme_dir[themes[i].type] = p;
+		}
+	}
+	themes_nr = k;
+	curtheme_dir[type] = NULL;
+
+	free(themes);
+	themes = new_themes;
+}
+
+int themes_lookup_idf(idf_t idf, const char *path, int type)
+{
+	char *p;
+	idff_t idf_dir;
+	int n = 0, i = 0;
+	struct theme *new_themes;
+	char *idf_de;
+	if (!idf)
+		return -1;
+	idf_dir = idf_opendir(idf, path);
+	if (!idf_dir)
+		return -1;
+	while ((idf_de = idf_readdir(idf_dir))) {
+		if (theme_lookup(idf_de, type))
+			continue;
+		if (!is_theme_idf(idf, path, idf_de))
+			continue;
+		n ++;
+	}
+	if (!n)
+		goto out;
+	idf_closedir(idf_dir); idf_dir = idf_opendir(idf, path);
+	if (!idf_dir)
+		return -1;
+	new_themes = realloc(themes, sizeof(struct theme) * (n + themes_nr));
+	if (!new_themes) {
+		idf_closedir(idf_dir);
+		return -1;
+	}
+	themes = new_themes;
+	while ((idf_de = idf_readdir(idf_dir)) && i < n) {
+		if (theme_lookup(idf_de, type))
+			continue;
+		if (!is_theme_idf(idf, path, idf_de))
+			continue;
+		p = getpath(path, idf_de);
+		themes[themes_nr].path = p;
+		themes[themes_nr].dir = strdup(idf_de);
+		themes[themes_nr].name = theme_name_idf(idf, p, idf_de);
+		themes[themes_nr].type = type;
+		themes[themes_nr].idf = 1;
+		themes_nr ++;
+		i ++;
+	}
+out:
+	idf_closedir(idf_dir);
+	themes_sort();
+	return 0;
+}
+
+int themes_lookup(const char *path, int type)
 {
 	char *p;
 	int n = 0, i = 0;
 	DIR *d;
 	struct dirent *de;
+	struct theme *new_themes;
 
 	if (!path)
 		return 0;
@@ -939,21 +1234,27 @@ int themes_lookup(const char *path)
 	if (!d)
 		return -1;
 	while ((de = readdir(d))) {
-		if (theme_lookup(de->d_name))
+		if (theme_lookup(de->d_name, type))
 			continue;
 		if (!is_theme(path, de->d_name))
 			continue;
 		n ++;
 	}
-		
-	rewinddir(d);
 	if (!n)
 		goto out;
-	themes = realloc(themes, sizeof(struct theme) * (n + themes_nr));
+	closedir(d); d = opendir(path);
+	if (!d)
+		return -1;
+	new_themes = realloc(themes, sizeof(struct theme) * (n + themes_nr));
+	if (!new_themes) {
+		closedir(d);
+		return -1;
+	}
+	themes = new_themes;
 	while ((de = readdir(d)) && i < n) {
 		/*if (de->d_type != DT_DIR)
 			continue;*/
-		if (theme_lookup(de->d_name))
+		if (theme_lookup(de->d_name, type))
 			continue;
 		if (!is_theme(path, de->d_name))
 			continue;
@@ -961,6 +1262,8 @@ int themes_lookup(const char *path)
 		themes[themes_nr].path = p;
 		themes[themes_nr].dir = strdup(de->d_name);
 		themes[themes_nr].name = theme_name(p, de->d_name);
+		themes[themes_nr].type = type;
+		themes[themes_nr].idf = 0;
 		themes_nr ++;
 		i ++;
 	}
@@ -984,37 +1287,74 @@ int themes_rename(void)
 	return 0;
 }
 
-static struct theme *theme_lookup(const char *name)
+struct theme *theme_lookup(const char *name, int type)
 {
 	int i;
 	if (!name || !*name) {
-		if (themes_nr == 1) 
+		if (themes_nr == 1 && themes[0].type == type) 
 			return &themes[0];
 		return NULL;
 	}
 	for (i = 0; i<themes_nr; i ++) {
-		if (!strlowcmp(themes[i].dir, name)) {
+		if (!strlowcmp(themes[i].dir, name) && themes[i].type == type) {
 			return &themes[i];
 		}
 	}
 	return NULL;
 }
 
-int game_theme_load(const char *name)
+int game_theme_load(const char *name, int type)
 {
+	struct theme *otheme = curtheme_loading;
 	struct theme *theme;
+	struct game *game;
 	char cwd[PATH_MAX];
 	int rc = -1;
 	int rel = theme_relative;
+
 	getdir(cwd, sizeof(cwd));
 	setdir(game_cwd);
-	theme = theme_lookup(name);
-	theme_relative = 0;
-	if (!theme || setdir(theme->path) || theme_load(THEME_FILE))
+
+	if (type == THEME_GLOBAL) {
+		theme_relative = 0;
+	} else {
+		game = game_lookup(curgame_dir);
+		if (!game)
+			return -1;
+		if (!game->idf)
+			setdir(game->path);
+		theme_relative = 1;
+	}
+
+	theme = theme_lookup(name, type);
+	if (!theme)
 		goto err;
+
+	if (theme->idf) /* cwd is always game_cwd */
+		strcpy(cwd, idf_getdir(game_idf));
+
+	curtheme_loading = theme;
+
+	if (theme->idf) {
+		if (idf_setdir(game_idf, theme->path))
+			goto err;
+		if (theme_load_idf(game_idf, THEME_FILE))
+			goto err;
+	} else {
+		if (setdir(theme->path))
+			goto err;
+		if (theme_load(THEME_FILE))
+			goto err;
+	}
 	rc = 0;
 err:
-	setdir(cwd);
+	curtheme_loading = otheme;
+
+	if (theme && theme->idf)
+		idf_setdir(game_idf, cwd);
+	else
+		setdir(cwd);
+
 	theme_relative = rel;
 	return rc;
 }
@@ -1022,14 +1362,26 @@ err:
 int game_theme_select(const char *name)
 {
 	struct theme *theme;
-	theme = theme_lookup(name);
+	theme = theme_lookup(name, THEME_GAME);
+	if (theme) {
+		curtheme_dir[THEME_GAME] = theme->dir;
+		game_cfg_save();
+		return 0;
+	}
+	theme = theme_lookup(name, THEME_GLOBAL);
 	if (!theme)
 		return -1;
-	curtheme_dir = theme->dir;
+	curtheme_dir[THEME_GLOBAL] = theme->dir;
 	return 0;
 }
 
 int game_default_theme(void)
 {
-	return game_theme_load(DEFAULT_THEME);
+	int rc;
+	rc = game_theme_load(DEFAULT_THEME, THEME_GLOBAL);
+	if (rc)
+		return rc;
+	if (opt_owntheme && themes_count(THEME_GAME) > 0)
+		rc = game_theme_load(DEFAULT_THEME, THEME_GAME);
+	return rc;
 }
