@@ -1,3 +1,27 @@
+/*
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include <limits.h>
 #include <pwd.h>
 #include <unistd.h>
@@ -8,6 +32,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
 
 #include "sdl_iconv.h"
 #include "internals.h"
@@ -45,7 +70,12 @@ char *game_locale(void)
 
 char *game_tmp_path(void)
 {
+	static time_t t = 0;
 	static char tmp[PATH_MAX]="/tmp/instead-games";
+	if (!t) {
+		t = time(NULL);
+		sprintf(tmp, "/tmp/instead-games-%ld", (unsigned long)t);
+	}
 	if (mkdir(tmp, S_IRWXU) && errno != EEXIST)
 		return NULL;
 	return tmp;
@@ -212,13 +242,13 @@ char *game_local_games_path(int cr)
 	strcpy(local_games_path, app);
 	if (cr) {
 		if (mkdir(local_games_path, S_IRWXU) && errno != EEXIST)
-        		return NULL;
-        }
-        strcat(local_games_path,"/games");
-        if (cr) { 
+			return NULL;
+	}
+	strcat(local_games_path,"/games");
+	if (cr) { 
 		if (mkdir(local_games_path, S_IRWXU) && errno != EEXIST)
-        		return NULL;
-        }
+			return NULL;
+	}
 	return local_games_path;
 }
 
@@ -347,3 +377,10 @@ char *dirpath(const char *path)
 	return fp;
 }
 #endif
+
+int is_absolute_path(const char *path)
+{
+	if (!path || !*path)
+		return 0;
+	return (*path == '/');
+}
