@@ -1,3 +1,27 @@
+/* 
+ * Copyright 2009-2016 Peter Kosyh <p.kosyh at gmail.com>, 2009 Ilya Ryndin
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction,
+ * including without limitation the rights to use, copy, modify, merge,
+ * publish, distribute, sublicense, and/or sell copies of the Software,
+ * and to permit persons to whom the Software is furnished to do so,
+ * subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+ * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+ * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+ * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #include <windows.h>
 #include <shlobj.h>
 #include <limits.h>
@@ -10,7 +34,6 @@
 #include "sdl_iconv.h"
 #include "internals.h"
 
-extern char *curgame;
 extern char *curgame_dir;
 
 static char local_games_path[PATH_MAX];
@@ -90,7 +113,7 @@ char *sdl_path(char *p)
 	return r;
 }
 
-char *app_dir( void );
+char *appdir( void );
 
 char *game_tmp_path(void)
 {
@@ -111,7 +134,7 @@ char *game_tmp_path(void)
 
 char *game_local_games_path(int cr)
 {
-	char *app = app_dir();
+	char *app = appdir();
 	if (!app)
 		return NULL;
 	snprintf(local_games_path, sizeof(local_games_path) - 1 , "%s/", app);
@@ -129,13 +152,13 @@ char *game_local_games_path(int cr)
 
 char *game_local_themes_path(void)
 {
-	snprintf(local_themes_path, sizeof(local_themes_path) - 1 , "%s/themes", app_dir());
+	snprintf(local_themes_path, sizeof(local_themes_path) - 1 , "%s/themes", appdir());
 	return local_themes_path;
 }
 
 char *game_local_stead_path(void)
 {
-	snprintf(local_stead_path, sizeof(local_stead_path) - 1 , "%s/stead", app_dir());
+	snprintf(local_stead_path, sizeof(local_stead_path) - 1 , "%s/stead", appdir());
 	return local_stead_path;
 }
 
@@ -152,33 +175,33 @@ char *home_dir( void )
 	return homedir;
 }
 #endif
-char *app_dir( void )
+char *appdir( void )
 {
-	static char appdir[PATH_MAX]="";
+	static char dir[PATH_MAX]="";
 #ifdef _LOCAL_APPDATA
 	if (appdata_sw)
-		strcpy(appdir, appdata_sw);
+		strcpy(dir, appdata_sw);
 	else {
-		strcpy(appdir, game_cwd);
-		strcat(appdir, "/appdata");
+		strcpy(dir, game_cwd);
+		strcat(dir, "/appdata");
 	}
-	if (!access(appdir, W_OK))
-		return appdir;
+	if (!access(dir, W_OK))
+		return dir;
 #endif
 
 	SHGetFolderPath( NULL, 
 		CSIDL_FLAG_CREATE | CSIDL_LOCAL_APPDATA,
 		NULL,
 		0, 
-		(LPTSTR)appdir );
-	unix_path(appdir);
-	strcat(appdir, "/instead");
-	return appdir;
+		(LPTSTR)dir );
+	unix_path(dir);
+	strcat(dir, "/instead");
+	return dir;
 }
 
 char *game_cfg_path( void )
 {
-	char *p = app_dir();
+	char *p = appdir();
 	if (!p)
 		return NULL;
 
@@ -197,8 +220,8 @@ char *game_cfg_path( void )
 
 char *game_save_path( int cr, int nr )
 {
-	char appdir[PATH_MAX];
-	char *p = app_dir();
+	char dir[PATH_MAX];
+	char *p = appdir();
 
 	if (!curgame_dir)
 		return NULL;
@@ -213,24 +236,24 @@ char *game_save_path( int cr, int nr )
         if (!p)
 		return NULL;
 
-	strcpy( appdir, p );
+	strcpy(dir,p);
 
-	if (cr && mkdir(appdir) && errno != EEXIST)
+	if (cr && mkdir(dir) && errno != EEXIST)
 		return NULL;
 
-	snprintf(save_path, sizeof(save_path) - 1 , "%s/saves", appdir);
+	snprintf(save_path, sizeof(save_path) - 1 , "%s/saves", dir);
 
 	if (cr && mkdir(save_path) && errno != EEXIST)
 		return NULL;
-	snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s", appdir, curgame_dir);
+	snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s", dir, curgame_dir);
 
 	if (cr && mkdir(save_path) && errno != EEXIST)
 		return NULL;
 
 	if (nr)
-		snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s/save%d", appdir, curgame_dir, nr);
+		snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s/save%d", dir, curgame_dir, nr);
 	else
-		snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s/autosave", appdir, curgame_dir);
+		snprintf(save_path, sizeof(save_path) - 1, "%s/saves/%s/autosave", dir, curgame_dir);
 
 	return save_path; 
 }
@@ -242,11 +265,16 @@ int debug_init(void)
 	SetConsoleTitle("Debug");
 	freopen("CON", "w", stdout); //Map stdout
 	freopen("CON", "w", stderr); //Map stderr
+	freopen("CON", "r", stdin); //Map stdin
 	return 0;
 }
 
 void debug_done()
 {
+	if (game_running) {
+		fprintf(stderr, "Press enter to close the console.\n");
+		fgetc(stdin);
+	}
 	FreeConsole();
 }
 #ifdef _USE_BROWSE
@@ -294,3 +322,14 @@ char *dirpath(const char *path)
 {
 	return (char*)path;
 }
+
+int is_absolute_path(const char *path)
+{
+	if (!path || !path[0])
+		return 0;
+	if (path[0] == '/' || path[0] == '\\')
+		return 1;
+	if (!path[1])
+		return 0;
+	return (path[1] == ':');
+}
